
###############################################################
# 
# Functional Genome-wide Association Studies
#
#
# Model: yi = mu + X*alpha + Z*beta + Xi*a + Zeta*d + ei
# Parameters: mu, alpha, beta, a, d, sigma
#
#  par$N: Observations: N
#  par$d1: discrete covariates: d1
#  par$d2: continuous covariates: d2
#  par$p:  SNP
#
# Y    - response, N*1
# X    - design matrix for discrete covariates, N*d1
# Z    - design matrix for continuous covariates, N*d2
# Xi   - design matrix for additive effects, N*p
# Zeta - design matrix for dominant effects, N*p
#
# References:
# 1. Jiahan Li et al. The Bayesian Lasso for Genome-wide Association 
#   Studies.
#
# History:
# 11/11/2010 Version 1.0
#
###############################################################

library(mvtnorm)

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# BLSApp.simu_test
#
# Abstract: a simpe method to call simulation
# 
# The result is saved into [simu_test_LS_$No$.rdata]
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

BLSApp.simu_test<-function( bX11Enable = FALSE )
{
	if (is.null( fGWAS_sys ) )
		fGWAS.start();

	st.par<- BLS.param();
	summary(st.par);

	st.dat<- BLS.simulate( st.par );
	summary(st.dat);
	if (bX11Enable) plot(st.dat);

	st.ret<- BLS.run( st.dat );
	summary(st.ret );
	if (bX11Enable) plot(st.ret);

	filename <- paste( "simu_test_bls_", fGWAS_sys$seq_id(),".rdata", sep="");
	save(st.par, st.dat, st.ret, file = filename );

	return( filename );
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# BLSApp.simu_mround
#
# Abstract: multiple simulation to get a steady result for 
#           model test and verify
# 
# The result is returned to "LS.ret" object
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

BLSApp.simu_mround<-function(nLoop, par=NULL)
{
	sm.par<-par;
	if (is.null(par))
		sm.par <- BLS.param();
		
	fGWAS_sys$task_start( 1, nLoop, paste("Execute the simulation ", nLoop, " times.") );	

	#---------multiple simulation/run------------
	rets<-list();
	for (i in 1:nLoop)
	{
		sm.dat <- BLS.simulate(sm.par);
		sm.dat$taskId<- i;

		fGWAS_sys$task_elapsed( i, paste("Round:", i, "/", nLoop, ",", "$SYS_PROMPT$") );
		r.run <- BLS.run(sm.dat, detail=FALSE);
		
		f.tmp <- fGWAS_sys$get_tmpfile("run_", ".rdata")
		save(r.run, file=f.tmp);
		rets[[i]] <- f.tmp;

		pheno_file <- sm.dat$pheno_file;
		chrs      = r.run$chrs;
		snp_names = r.run$snp_names;
	}

	maxIter <- fGWAS.get_value("max_iter", def=2000);
	nDel <- as.integer(maxIter/10);

	#-------------computing a and d, h2----------
	me_as<-c();	
	me_ds<-c();	
	me_h2<-c();
	for (i in 1:nLoop)
	{
		load(file=rets[[i]] );
		unlink(rets[[i]]);

		me_a<-c();
		y<-r.run$r_a[-(1:nDel),];
		for (j in 1:length(y[1,]))
		{
			me_a<-c(me_a, median(y[,j]));
		}
		me_as<-rbind(me_as, me_a);	

		me_d<-c();
		y<-r.run$r_d[-(1:nDel),];
		for (j in 1:length(y[1,]))
		{
			me_d<-c(me_d, median(y[,j]));
		}
		me_ds<-rbind(me_ds, me_d);	

		me_h2<-rbind(me_h2, r.run$r_h2);	
		gc();
		
		cat("Memory.size=", memory.size(), "\n");
	}

	fGWAS_sys$task_stop( "The simulation test is done" );	

	me_d <- colMeans(me_ds);
	me_a <- colMeans(me_as);
	r_h2 <- colMeans(me_h2);

	#-------------marshel result--------------
	sm.ret <- list(name= "LS.ret.M2",
 	            pheno_file = pheno_file,
	            chrs = chrs,
	            snp_names = snp_names,
	            me_a = me_a,
	            me_d = me_d,
	            r_h2 = r_h2);
	class(sm.ret)<-"LS.ret";
	
	return(sm.ret);
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# BLSApp.onestep
#
# Abstract: a simpe way to run a simple data set
# 
# The result is saved into "outfile.rdata"
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

BLSApp.onestep<-function(tped_file, tfam_file, sPheno_csv, file.rdata=NULL, file.report=NULL, bX11Enable = FALSE )
{
	if (is.null( fGWAS_sys ) )
		fGWAS.start();

	onestep.dat<- BLS.load_plink(tped_file, tfam_file, sPheno_csv);
	summary(onestep.dat);
	if(bX11Enable) plot(onestep.dat);

	onestep.ret1<- BLS.run( onestep.dat, detail=FALSE );
	summary(onestep.ret1);
	if(bX11Enable) plot(onestep.ret1);

	onestep.ret2<- BLS.refit( onestep.dat, onestep.ret1 );
	summary(onestep.ret2);
	if(bX11Enable) plot(onestep.ret2);

	if (is.null(file.rdata))
		file.rdata <- fGWAS.get_subfile( sPheno_csv, paste("-", fGWAS_sys$seq_id(),".bls.rdata", sep="") );

	save( onestep.dat, onestep.ret1, onestep.ret2, file = file.rdata );
	
	if (is.null(file.report))
		file.report <- fGWAS.get_subfile( sPheno_csv, paste("-", fGWAS_sys$seq_id(),".bls.rpt.pdf", sep="") );

	BLSApp.report(file.report, onestep.dat, onestep.ret1, onestep.ret2 )
	
	return( file.rdata );
}

BLSApp.report<-function( pdf.file, dat, res1, res2, options=list() )
{
	Report.new( pdf.file, options );
	Report.title( "fGWAS Report(Blasso Model)", "fGWAS Report(BLS)", "http://statgen.psu.edu" );
	Report.par( "dat.file", dat$pheno_file);
	
	Report.AddHeadline( "Data Summary", level=1 );
	frep.report_dat(dat);

	Report.AddHeadline( "Effect Estimation", level=1 );
	
	if ( !is.null(res1) )   
	{
		pdata <- c();
		pdata <- cbind( pdata, res1$snp_chrs);
		pdata <- cbind( pdata, res1$snp_pos);
		pdata <- cbind( pdata, res1$me_a);
		pdata <- cbind( pdata, res1$me_d);
		pdata <- cbind( pdata, res1$r_h2);
		colnames(pdata)<-c("chr", "pos", "a", "d", "h2");

		frep.report_adh2_profile( pdata, 3) ;
		Report.AddParaBreak();
	}

	Report.AddHeadline( "Significant SNPs", level=1 );
	
	if ( !is.null(res2) )	  
	{
		r <- proc_select_sigpos(res2);
		aList<-r$a;
		dList<-r$d;

		sig.dat <- data.frame(cbind(
					name= res2$snp_names[aList],
					chr = res2$snp_chrs[aList], 
					pos = res2$snp_pos[aList],
					add = sprintf( "%.3f", res2$me_a[aList] ), 
					dom = sprintf( "%.3f", res2$me_d[aList] ), 
					H2  = sprintf( "%.3f", res2$r_h2[aList] ) ) );

		Report.AddTable( sig.dat, 
							 title = "Significant Additive SNPs",
							 frame = "wang",  
							 col.names= c("Name.", "Chr.", "Pos.", "Add.", "Dom.", "H2" ),
							 col.width= c( 1,      0.5,    1,      1,      1,      1)*254,
							 col.align= c("L",     "L",    "R",    "R",    "R",    "R"),
							 offset.x = 50, 
							 max.show = 20);

		Report.AddParaBreak();

		sig.dat <- data.frame(cbind(
					name= res2$snp_names[dList],
					chr = res2$snp_chrs[dList], 
					pos = res2$snp_pos[dList],
					add = sprintf( "%.3f", res2$me_a[dList] ), 
					dom = sprintf( "%.3f", res2$me_d[dList] ), 
					H2  = sprintf( "%.3f", res2$r_h2[dList] ) ) );

		Report.AddTable( sig.dat, 
							 title = "Significant Dominant SNPs",
							 frame = "wang",  
							 col.names= c("Name.", "Chr.", "Pos.", "Add.", "Dom.", "H2" ),
							 col.width= c( 1,      0.5,    1,      1,      1,      1)*254,
							 col.align= c("L",     "L",    "R",    "R",    "R",    "R"),
							 offset.x = 50, 
							 max.show = 20);

	}

	Report.Output( pdf.file );
	return(pdf.file);
}


