
###############################################################
# 
# Functional Genome-wide Association Studies
#
#
# Model: yi = mu + X*alpha + Z*beta + Xi*a + Zeta*d + ei
# Parameters: mu, alpha, beta, a, d, sigma
#
#  par$N: Observations: N
#  par$d1: discrete covariates: d1
#  par$d2: continuous covariates: d2
#  par$p:  SNP
#
# Y    - response, N*1
# X    - design matrix for discrete covariates, N*d1
# Z    - design matrix for continuous covariates, N*d2
# Xi   - design matrix for additive effects, N*p
# Zeta - design matrix for dominant effects, N*p
#
# References:
# 1. Jiahan Li et al. The Bayesian Lasso for Genome-wide Association 
#   Studies.
#
# History:
# 11/11/2010 Version 1.0
#
###############################################################

library(mvtnorm);

#--------------------------------------------------------------
# public: BLS.load_plink
#
# load snp data exported by plink and CSV phenotype data for this 
# fGWAS model
# 
# input:
#  tped_file: exported by plink
#  tfam_file: exported by plink
#  tpheno_file: CSV file
#
# Batch: 
#    plink --file {data_file} --recode --chr {1} --out {oufile} --transpose --noweb
#
#--------------------------------------------------------------
BLS.load_plink<-function( tped_file, tfam_file, tpheno_file )
{
	dat <- fdat.load_plink( tped_file, tfam_file, tpheno_file );
	return( dat );
}

#--------------------------------------------------------------
# BLS.param
#
# Create a param object for the simulation, "fGWAS.par.nonlong" is returned
#--------------------------------------------------------------
BLS.param<-function()
{
	return(fdat.nonlong_param());
}

#--------------------------------------------------------------
# summary(fGWAS.par.nonlong)
#
# summarize a "fGWAS.par.nonlong" object
#--------------------------------------------------------------
summary.fGWAS.par.nonlong <- function( par )
{
	cat( fdat.summary_nonlong_param(par) );
	invisible();
}

#--------------------------------------------------------------
# BLS.simulate
#
# Simulate a data set for fGWAS, including SNP(genotype), Y(phenotype)
# covarite( X for discrete and Z for continuous )
#
# Input  
#       parameter object for Backcross(fGWAS.par.nonlong)
# output:
#       data object( fGWAS.dat.nonlong )
#--------------------------------------------------------------
BLS.simulate<-function( par )
{
	return( fdat.simulate_nonlong(par) );
}

#--------------------------------------------------------------
# summary(fGWAS.dat.nonlong)
#
# summarize the important information for the simulate data or 
# real data.
#
# input : "fGWAS.dat.nonlong" object.
#--------------------------------------------------------------
summary.fGWAS.dat.nonlong <- function( dat )
{
	cat( fdat.summary_dat( dat ) );
	
	plot.fGWAS.dat.nonlong( dat, bOutputDoc=TRUE );

	invisible()
}

#--------------------------------------------------------------
# private: plot(fGWAS.dat.nonlong)
#
# input : "fGWAS.dat.nonlong" object.
#--------------------------------------------------------------
plot.fGWAS.dat.nonlong <- function( dat, bOutputDoc=FALSE )
{
	strFile = "";
	if ( bOutputDoc )
	{
		strFile <-  fGWAS.get_subfile( dat$pheno_file, paste("-", fGWAS_sys$seq_id(),".bls.dat.pdf", sep="") )
		pdf( strFile );
	}
	
	x<-hist(dat$pheno_y, breaks=20);
	xlim = range(x$breaks);
	ylim <- range(x$counts, 0);
	nB <- length(x$breaks);
	plot(1:10,1:10, xlim=xlim, ylim=ylim, type="n", xlab="", ylab="");
    rect(x$breaks[-nB], 0, x$breaks[-1L], x$counts,  col = "blue", border = "black");
	title(main = "Histogram  of phenotype", ylab = "Frequency", xlab = "Pheno(Y)" );

	if ( bOutputDoc )
	{
		dev.off();
		str <- paste("*1:The figure 1 for all individuals is saved to ", strFile, ".\n", sep="");
		return (str);
	}
}

#--------------------------------------------------------------
# BLS.run
#
# 1)Supervised PCA 
# 2)Perform MCMC algorithm
# 3)summarize a/d/h2
# 4)Marshal result
#--------------------------------------------------------------
BLS.run<-function( dat, refit=FALSE, detail=TRUE )
{
	seed.n <- Sys.getpid() + as.numeric(format(Sys.time(), "%H%M%S"))
	seed.n <- 100;
	cat("!!!Set.seed=", seed.n, "\n");
	set.seed( seed.n );

	#-------------supervised PCA-------------
	if (refit)
		dat$prec_y <- dat$pheno_y
	else
	{
		dat$prec_y <- proc_prec( dat );
		gc();
	}

	#-------------MCMC algorithm-------------
	#Data Feeding and Cleaning, load univariate response 
	
	file.a       <- c();
	file.d       <- c();
	file.tau2    <- c();
	file.lambda2 <- c();
	file.tau_st2 <- c();
	file.lambda_st2 <- c();
	file.sigma2  <- c();

	p <- length(dat$snps[,1]);
	res <- c();
	i.step<-1000;

	fGWAS_sys$task_start( 1, ceiling(p/i.step ) );
	for(i in 1:ceiling(p/i.step ) )
	{
		fGWAS_sys$task_elapsed( i, paste( "No.", i, " SNP group is entering MCMC procedure", sep="") );
		if (i*i.step<=p)
		{
			Xi.sect   <- dat$snps[((i-1)*i.step+1):(i*i.step),] - 1;
			Zeta.sect <- 1-abs(Xi.sect);
		}
		else
		{
			Xi.sect   <- dat$snps[((i-1)*i.step+1):p,] -1;
			Zeta.sect <- 1-abs(Xi.sect);
		}

		res.sect <- proc_mcmc(dat$prec_y, dat$covar_x, dat$covar_z, t(Xi.sect), t(Zeta.sect), refit, taskId=dat$taskId, sectId=i );

		# save the temporary results into RDATA file to save memory
		r.a       <- res.sect$r_a;
		r.d       <- res.sect$r_d;
		r.tau2    <- res.sect$r_tau2;
		r.lambda2 <- res.sect$r_lambda2;
		r.tau_st2 <- res.sect$r_tau_st2;
		r.lambda_st2 <- res.sect$r_lambda_st2;
		r.sigma2  <- res.sect$r_sigma2;

		f.tmp <- fGWAS_sys$get_tmpfile("a_", ".rdata")
		save( r.a,      file= f.tmp ); 
		file.a <- c( file.a, f.tmp );

		f.tmp <- fGWAS_sys$get_tmpfile("d_", ".rdata") 
		save( r.d,      file= f.tmp ); 
		file.d <- c( file.d, f.tmp);
	
		if (detail)
		{
			f.tmp <- fGWAS_sys$get_tmpfile("tau2_", ".rdata") 
			save( r.tau2,   file= f.tmp )
			file.tau2 <- c( file.tau2, f.tmp );

			f.tmp <- fGWAS_sys$get_tmpfile("lambda2_", ".rdata") 
			save( r.lambda2,file= f.tmp ); 
			file.lambda2 <- c( file.lambda2, f.tmp);

			f.tmp <- fGWAS_sys$get_tmpfile("tau_st2_", ".rdata") 
			save( r.tau_st2,file= f.tmp ); 
			file.tau_st2 <- c( file.tau_st2, f.tmp);

			f.tmp <- fGWAS_sys$get_tmpfile("lambda_st2_", ".rdata") 
			save( r.lambda_st2, file= f.tmp ); 
			file.lambda_st2 <- c(file.lambda_st2, f.tmp);

			f.tmp <- fGWAS_sys$get_tmpfile("sigma2_", ".rdata") 
			save( r.sigma2, file= f.tmp ); 
			file.sigma2  <- c(file.sigma2, f.tmp);
		}

		gc();
	}

	#-------------load temporary results for a/d-------------
	r_a <- c();
	r_d <- c();
	
	for( f.tmp in file.a)
	{
		load( f.tmp);
		r_a <- cbind(r_a,  r.a );
		unlink( f.tmp )
	}

	for( f.tmp in file.d)
	{
		load( f.tmp);
		r_d <- cbind(r_d,  r.d );
		unlink( f.tmp )
	}
	
	#-------------computing a(additive)-------------
	nDel <- as.integer(fGWAS.get_value("max_iter", def=2000)/10);
	y <- r_a[-(1:nDel),];
	me_a<-c();
	for (i in 1:length(y[1,]))
	{
		me_a<-c(me_a, median(y[,i]));
	}

	#-------------computing d(dominant)-------------
	y <- r_d[-(1:nDel),];
	me_d<-c();
	for (i in 1:length(y[1,]))
	{
		me_d<-c(me_d, median(y[,i]));
	}

	#-------------computing h2--------------
	Y  <- dat$prec_y;
    N  <- length(Y);
    p1 <- c();
	for (j in 1:p )
	{
		tmp <- dat$snps[j,];
		p1  <- c(p1, (2*length(which(tmp==1))+length(which(tmp==0)))/2/N );
	}

	p0 <- 1 - p1;
	var_a <- 2*p1*p0*(me_a + (p1-p0)*me_d)^2;
	var_d <- 4*(p1^2)*(p0^2)*(me_d^2);
	var_d <- var_d*3;
	r_h2  <- (var_a + var_d)/var(Y)*100;

	#-------------if detail is neccesary, load all data--------------

	r_tau2    <- c();
	r_lambda2 <- c();
	r_tau_st2 <- c();
	r_lambda_st2 <- c();
	r_sigma2  <- c();

    if (detail)
    {
		for( f.tmp in file.tau2)
		{
			load( f.tmp);
			r_tau2 <- cbind(r_tau2,  r.tau2 );
			unlink( f.tmp )
		}

		for( f.tmp in file.lambda2)
		{
			load( f.tmp);
			r_lambda2 <- cbind(r_lambda2,  r.lambda2 );
			unlink( f.tmp )
		}


		for( f.tmp in file.tau_st2)
		{
			load( f.tmp);
			r_tau_st2 <- cbind(r_tau_st2,  r.tau_st2 );
			unlink( f.tmp )
		}

		for( f.tmp in file.lambda_st2)
		{
			load( f.tmp);
			r_lambda_st2 <- cbind(r_lambda_st2,  r.lambda_st2 );
			unlink( f.tmp )
		}

		for( f.tmp in file.sigma2)
		{
			load( f.tmp);
			r_sigma2 <- cbind(r_sigma2,  r.sigma2 );
			unlink( f.tmp )
		}
	}


	fGWAS_sys$task_stop();

	#-------------marshal result--------------
	
	if (detail)
		ret<-list(
			model     = "BLS",
			name      = "bls.vs",
			taskId    = fGWAS_sys$seq_id(),
			pheno_file= dat$pheno_file,
			snp_chrs  = dat$snps.info[,1,drop=FALSE],
			snp_pos   = dat$snps.info[,2,drop=FALSE],
			snp_names = rownames(dat$snps),
			detail	  = detail, 	
			r_a       = r_a,
			r_d       = r_d,
			r_tau2    = r_tau2,
			r_lambda2 = r_lambda2,
			r_tau_st2 = r_tau_st2,
			r_lambda_st2 = r_lambda_st2,
			r_sigma2  = r_sigma2,
			me_a      = me_a,
			me_d      = me_d,
			r_h2      = r_h2)
	else
		ret<-list(
			model     = "BLS",
			name      = "bls.vs",
			pheno_file= dat$pheno_file,
			snp_chrs  = dat$snps.info[,1,drop=FALSE],
			snp_pos   = dat$snps.info[,2,drop=FALSE],
			snp_names = rownames(dat$snps),
			detail	  = detail, 	
			r_a       = r_a,
			r_d       = r_d,
			me_a      = me_a,
			me_d      = me_d,
			r_h2      = r_h2)
	
	class(ret)<-"fGWAS.ret.bls";
	
	return(ret);		
}

#--------------------------------------------------------------
# proc_prec
#
# Supervised PCA
#--------------------------------------------------------------
proc_prec<-function( dat )
{
	y   <- dat$pheno_y;
	Xi  <- dat$snps[,] - 1; 
	Zeta<- 1- abs(Xi);
	tmp <- which(as.integer(Xi)==9);
	if (length(tmp)>0 )
	    stop("There is still missing genotypes after imputing!");

	p  <- length(Xi[,1]);
	reg_coeff <- array(0, p);
	reg_num <- as.integer(0.03*p);
	if (reg_num<=2)
		reg_num = 2;

	for (i in 1:p) 
	   	reg_coeff[i] <- lm( y~Xi[i,])$coeff[2];
	sorted_coeff<- sort(abs(reg_coeff), decreasing=T );
	#--reduced data matrix;
	reduced_a <- Xi[which(abs(reg_coeff)>=sorted_coeff[reg_num]), ] 

	p <- length(Zeta[,1]);
	reg_coeff <- array(0, p)
	for (i in 1:p) 
	   reg_coeff[i] <- lm(y~Zeta[i,])$coeff[2];
	sorted_coeff <- sort(abs(reg_coeff),decreasing=T);
	#--reduced data matrix;
	reduced_d <- Zeta[which(abs(reg_coeff)>=sorted_coeff[reg_num]), ] 

	reduced_pred = t(rbind(reduced_a,reduced_d));
	#--PCA
	reduced_pred_pca <- prcomp(reduced_pred, scale = TRUE);
	
	m<-fGWAS.get_value("reduce_order", def=3);
	reduced_pca1 <- predict(reduced_pred_pca)[,c(1:m)] ;
	#--obtain preconditioned response
	prec_y <- lm(y~reduced_pca1)$fitted.values;

	##output to console
	cat("** Supervised principle component analysis is performed successfully.\n");
	
	return(prec_y);
}


#--------------------------------------------------------------
# proc_mcmc
#
# Input 
# Y    - response, N*1
# X    - design matrix for discrete covariates, N*d1
# Z    - design matrix for continuous covariates, N*d2
# Xi   - design matrix for additive effects, N*p
# Zeta - design matrix for dominant effects, N*p
#
# Model: yi = M*mu + X*alpha + Z*beta + Xi*a + Zeta*d + ei
# Parameters: mu, alpha, beta, a, d, sigma
#
# of Observations: N
# of SNP: p
# of discrete covariates: d1
# of continuous covariates: d2
#--------------------------------------------------------------
proc_mcmc<-function( Y, X, Z, Xi, Zeta, refit, taskId=-1, sectId=-1  )
{
	maxIter <- as.integer( fGWAS.get_value("max_iter", def=2000) );
	mcmc_hint <- as.integer( fGWAS.get_value("mcmc_hint", def=100) );

	#--check dimensions
	N <- length(Y);
	p <- length(Xi[1,]);
	M <- rep(1, N );
	d1<- 1;
	d2<- 1;

	#--Hyper-parameters for alpha, beta, a, d, and lambda2
	sigmaMu    <- diag(rep(1, 1), nrow=1);
	sigmaAlpha <- diag(rep(1, d1), nrow=d1);
	sigmaBeta  <- diag(rep(1, d2), nrow=d2);
	gammaTau   <- 1;
	deltaTau   <- 1;
	gammaTau_st <- 1;
	deltaTau_st <- 1;

	#--Initializing all parameters: mu, alpha, beta, sigma2, a, d, ...
	#Y  <- Y - mu;
	sigma2 <- var( Y - mean(Y) );
	alpha <- rep(0, d1);
	beta  <- rep(0, d2);
	#--additive effect and its hyperparameters
	a     <- t(rep(0,p)); 
	tau2  <- rep( 1, p);
	lambda2 <- 1;
	#--dominant effect and its hyperparameters
	d     <- t(rep(0,p)); 
	tau_st2 <- rep(1, p);
	lambda_st2 <- 1;

	r_sigma2  <- c();
	r_mu      <- c();
	r_alpha   <- c();
	r_beta    <- c();
	r_a       <- c(); 
	r_tau2    <- c();
	r_lambda2 <- c();
	r_d       <- c(); 
	r_tau_st2 <- c();
	r_lambda_st2 <- c();

	##output to console
	cat("** MCMC sampling procedure will start", "Section:", sectId, ".\n");

	fGWAS_sys$task_start( 1, maxIter );

	#--begin Gibbs sampling
	for (round in 1:maxIter)
	{	
		if ((round%%mcmc_hint==0) || (round<10 ) )
		{
			msg <- paste("Section:", sectId, " Task.id:", taskId, " Round:", round, " Estimation: $SYS_PROMPT$", sep="");
			fGWAS_sys$task_elapsed( round, msg=msg);
		}

		muVar <- 1/(t(M)%*%M/sigma2+1/sigmaMu);
		muMu  <- muVar*(t(M)%*%(Y- X*alpha- Z*beta-Xi%*%t(a)-Zeta%*%t(d))/sigma2);
		mu <- rnorm(1, mean=muMu, sd=sqrt(muVar));

		#--updating alpha
		if (length(X) > 1 )
		{
			alphaVar <- 1/(t(X)%*%X/sigma2+1/sigmaAlpha);
			alphaMu  <- alphaVar*(t(X)%*%(Y-M*mu-Z*beta-Xi%*%t(a)-Zeta%*%t(d))/sigma2);
			alpha <- rnorm(1, mean=alphaMu, sd=sqrt(alphaVar));
		}

		#--updating beta
		if (length(Z) > 1 )
		{
			betaVar <- 1/(t(Z)%*%Z/sigma2+1/sigmaBeta);
			betaMu  <- betaVar*(t(Z)%*%(Y-M*mu-X*alpha-Xi%*%t(a)-Zeta%*%t(d))/sigma2);
			beta <- rnorm(1, mean=betaMu, sd=sqrt(betaVar));
		}

	    #--Updating additive effects: a, for each SNP 
	    tmp <- c();
	    for (j in 1:p)
	        tmp <- c( tmp, t(Xi[,j])%*%Xi[,j] );
		
		if (!refit) tmp  <- (tmp + 1./tau2);

	    tmp3 <- Y - M*mu- X*alpha - Z*beta - Zeta%*%t(d);
	    aVar <- sigma2/tmp;

	    for (j in 1:p)
	    {
	    	aMu_j <- t(Xi[,j])%*%( tmp3 - Xi%*%t(a) + Xi[,j]*a[j] )/tmp[j];
	        a[j] <- rnorm( 1, mean=aMu_j, sd=sqrt( aVar[j] ) );
	    }

	    #--Updating tau2 underlying a in non-refit status
		if (!refit)
	    {
	    	for (j in 1:p)
	    	{
	        	InvTau2_1 <- sqrt(lambda2 * sigma2)/abs(a[j]);
	        	tau2[j] <- 1/func_invGau(InvTau2_1, lambda2);
	    	}
	
	    	#--Updating lambda2 underlying tau2
	    	lambda2 <- rgamma( 1, p+gammaTau, scale=sum(tau2)/2 + deltaTau );

		}
	
	    #--Updating dominant effects: d, for each SNP
	    tmp <- c();
	    for (j in 1:p)
	        tmp <- c(tmp, t(Zeta[,j])%*%Zeta[,j] );

	    if (!refit) tmp  <- (tmp + 1./tau_st2);

	    tmp3 <- Y - M*mu- X*alpha - Z*beta - Xi%*%t(a);
	    dVar <- sigma2/tmp;
	    for (j in 1:p)
	    {
	    	dMu_j <- t(Zeta[,j])%*%(tmp3-Zeta%*%t(d)+Zeta[,j]*d[j])/tmp[j];
	        d[j]  <- rnorm( 1, mean=dMu_j, sd=sqrt(dVar[j]));
	    }
    
	    #--Updating tau_st2 underlying d in non-refit status
		if (!refit) 
	    {
	    	for (j in 1:p)
	    	{
				InvTau2_1 = sqrt(lambda_st2*sigma2)/abs(d[j]);
				tau_st2[j] = 1/func_invGau(InvTau2_1, lambda_st2);
	    	}
	
	    	#--Updating lambda_star2 underlying tau_st2
	    	lambda_st2 <- rgamma( 1, p + gammaTau_st, scale=sum(tau_st2)/2+deltaTau_st );
		}

		#--Updating residual variance: sigma2 
	    tmp0   <- Y - M*mu - X*alpha - Z*beta - Xi%*%t(a) - Zeta%*%t(d);
	    # scale parameter
	    tmp    <- t(tmp0)%*%tmp0/N; 
	    
	    sigma2 <- 1/rchisq(1,N)*N*tmp;

		cat("round=", round, "mu=", mu, "alpha=", alpha, "beta=", beta, "lambda=", lambda2, "lambda*=", lambda_st2, "sigma2=", sigma2, " \n");

	    r_sigma2  <- rbind( r_sigma2, sigma2);
		r_a       <- rbind( r_a, a); 
		r_d       <- rbind( r_d, d); 
		r_mu      <- rbind( r_mu, mu);
		r_alpha   <- rbind( r_alpha, alpha);
		r_beta    <- rbind( r_beta, beta);
		r_tau2    <- rbind( r_tau2, tau2);
		r_lambda2 <- rbind( r_lambda2, lambda2);
		r_tau_st2 <- rbind( r_tau_st2, tau_st2);
		r_lambda_st2 <- rbind( r_lambda_st2, r_lambda_st2);
	}


	ret<-list(
		r_a       = r_a,
		r_d       = r_d,
		r_mu      = r_mu,
		r_alpha   = r_alpha,
		r_beta    = r_beta,
		r_tau2    = r_tau2,
		r_lambda2 = r_lambda2,
		r_tau_st2 = r_tau_st2,
		r_lambda_st2 = r_lambda_st2,
		r_sigma2  = r_sigma2 );
	
	##output to console
	fGWAS_sys$task_stop("** MCMC sampling procedure is done successfully.");


	return( ret )
}


func_invGau<-function(theta,chi)
{
	# squared normal, i.e., chi-square with df=1
	chisq1<- rnorm(1, 0, 1)^2; 
	y1    <- theta + 0.5*theta/chi * ( theta*chisq1 - sqrt(4*theta*chi*chisq1 + theta^2*chisq1^2) );
	y2    <- theta^2/y1;
	out_1 <- runif(1) < theta/(theta+y1);
	value <- out_1*y1+(1-out_1)*y2;
	
	return(value);
}

#--------------------------------------------------------------
# summary(fGWAS.ret.bls)
#
# 1)list the significant postion
# 2)draw an overview graph for additive, dominant and heritability
#
# input : "fGWAS.ret.bls" object.
#--------------------------------------------------------------

summary.fGWAS.ret.bls<-function(res)
{
	aList <- 1:length(res$me_a);
	dList <- 1:length(res$me_d);
	if (res$name!="bls.refit")
	{
		r <- proc_select_sigpos(res);
		aList<-r$a;
		dList<-r$d;
	}

	stra1 <- sprintf("The additive significant SNPs:\n");
	stra2 <- sprintf("----------------------------------------------\n");
	stra3 <- sprintf("%5s %12s %10s %7s %7s\n", "Chr", "Name", "Position", "Additive", "H2");
	stra4 <- c();
	for(i in 1:length(aList))
	{
		stra4 <- c(stra4, sprintf("%5.0f %12s %10.0f %7.3f %7.3f\n", res$snp_chrs[aList[i]], res$snp_names[aList[i]], res$snp_pos[aList[i]], res$me_a[aList[i]], res$r_h2[aList[i]]) );
	}
	stra5 <- sprintf("----------------------------------------------\n");


	strd1 <- sprintf("The dominant significant SNPs:\n");
	strd2 <- sprintf("----------------------------------------------\n");
	strd3 <- sprintf("%5s %12s %10s %7s %7s\n", "Chr", "Name", "Position", "Dominant", "H2");
	strd4 <- c();
	for(i in 1:length(dList))
	{
		strd4 <- c(strd4, sprintf("%5.0f %12s %10.0f %7.3f %7.3f\n", res$snp_chrs[dList[i]], res$snp_names[dList[i]], res$snp_pos[dList[i]], res$me_d[dList[i]], res$r_h2[dList[i]]) );
	}
	strd5 <- sprintf("----------------------------------------------\n");
	
	strX<- plot.fGWAS.ret.bls( res, bOutputDoc=TRUE );
	
	str <- paste(c(stra1,stra2,stra3,stra4,stra5,"\n", 
		 		 strd1,strd2,strd3,strd4,strd5,"\n", strX), collapse ="");
	
	cat(str);
}

proc_select_sigpos<-function(res)
{
	relRatio <- as.numeric(fGWAS.get_value("RelRatio", 0.25));

	tThres<-max(res$me_a);
	aList <- which(res$me_a > tThres*relRatio);
	if (length(aList)==0 || length(aList)>10 )	
	{
		y<-order(res$me_a, decreasing=TRUE);
		if ( length(aList)>10) 
			n.Top <- 10
		else
			n.Top <- length(res$me_a);

		aList <- y[1:n.Top];
	}


	tThres<-max(res$me_d);
	dList <- which(res$me_d > tThres*relRatio);
	if (length(dList)==0 || length(dList)>10)	
	{
		y<-order(res$me_d, decreasing=TRUE);

		if ( length(dList)>10) 
			n.Top <- 10
		else
			n.Top<-length(res$me_d);

		dList <- y[1:n.Top];
	}	
	
	return (list(a=aList, d=dList));
}

#--------------------------------------------------------------
# plot( fGWAS.ret.bls )
#
# draw an overview graph for additive, dominant and heritability
#
# input : "fGWAS.ret.bls" object.
#--------------------------------------------------------------

plot.fGWAS.ret.bls<-function(res, bOutputDoc=FALSE)
{
	strFile = "";
	if ( bOutputDoc )
	{
		strFile <- fGWAS.get_subfile( res$pheno_file, paste("-", fGWAS_sys$seq_id(),".bls.res.pdf", sep="") );
		pdf( strFile );
	}
	
	pdata <- c();
	pdata <- cbind( pdata, res$snp_chrs);
	pdata <- cbind( pdata, res$snp_pos);
	pdata <- cbind( pdata, res$me_a);
	pdata <- cbind( pdata, res$me_d);
	pdata <- cbind( pdata, res$r_h2);

	par(mar=c(4.5, 4, 0.5, 2) + 0.1);
	plot.new();
  	par(mfrow=c(3,1));
	
	n.col <- length(pdata[1,]);

	par(mfg=c(1, 1));
	fplot.single_adh2( pdata[,n.col-2], res$snp_chrs, "Estimated additive effect");
	par(mfg=c(2, 1));
	fplot.single_adh2( pdata[,n.col-1], res$snp_chrs, "Estimated dominant effect");
	par(mfg=c(3, 1));
	fplot.single_adh2( pdata[,n.col], res$snp_chrs, "Heritability");

	if ( bOutputDoc )
	{
		dev.off();
		str <- paste("*1:The figure 1 for all SNPs is saved to ", strFile, ".\n", sep="");
		return (str);
	}
}

#--------------------------------------------------------------
# BLS.refit
#
#
# input: fGWAS.dat.nonlong and fGWAS.dat.bls(from BLS.run())
#--------------------------------------------------------------

BLS.refit<-function(org_dat, org_res)
{
	##output to console
	cat("** Significant positions is being selected for Beyansian Lasso Model again\n");

	r <- proc_select_sigpos(org_res);
	rList <- unique(c(r$a, r$d));
	
	dat<-list(
		model        = "BLS",
		sample_times = 1,
		sample_n     = 0,
		sample_p     = 0,
		pheno_file   = org_dat$pheno_file,
		geno_file    = org_dat$geno_file,
		raw_snps     = NULL,
		snps         = NULL,
		subj_ids     = org_dat$subj_ids,
		pheno_y      = org_dat$pheno_y,
		covar_x      = org_dat$covar_x,
		covar_z      = org_dat$covar_z);
	
	dat$snps.info<- org_dat$snps.info[rList,,drop=FALSE];
	dat$snps<- org_dat$snps[rList,,drop=FALSE];
	dat$sample_n <- length(dat$subj_ids);
	dat$sample_p <- length(dat$snps[,]);
	
	ret <- BLS.run(dat, refit=TRUE);

	ret$name <- "bls.refit";
	
	cat("** Refit procedure is done successfully\n");
	
	return(ret);		
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# BLS.merge_result
#
# Abstract: merge multiple chrmosome result into one "LS.ret" object
# 
# input:    Multiple "LS.ret" object
# output:   LS.ret object
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

BLS.merge_result<-function(...)
{
	res <- list(...);
	
	for (i in 1:length(res))
		if (class(res[[i]])!="LS.ret")
			stop(sprintf("The No.%s parameter is not a 'LS.ret' object", i));
	
	ret<-list(name = "bls.merge",
	          pheno_file= res[[1]]$pheno_file,
	  		  snp_chrs  = NULL,
		      snp_pos   = NULL,
	          snp_names = NULL,
	          me_a      = NULL,
	          me_d      = NULL,
	          r_h2      = NULL );

	class(ret)<-"fGWAS.ret.bls";
	          
	ret$pheno_file <- res[[1]]$pheno_file;
	ret$chrs <- res[[1]]$chrs; 
	for (i in 2:length(res))
		ret$chrs <- c(ret$chrs, res[[i]]$chrs);

	ret$snp_names <- res[[1]]$snp_names; 
	for (i in 2:length(res))
		ret$snp_names <- c(ret$snp_names, res[[i]]$snp_names);

	ret$snp_chrs <- res[[1]]$snp_chrs; 
	for (i in 2:length(res))
		ret$snp_chrs <- c(ret$snp_chrs, res[[i]]$snp_chrs);

	ret$snp_pos <- res[[1]]$snp_pos; 
	for (i in 2:length(res))
		ret$snp_pos <- c(ret$snp_pos, res[[i]]$snp_pos);

	ret$me_a <- res[[1]]$me_a; 
	for (i in 2:length(res))
		ret$me_a <- c(ret$me_a, res[[i]]$me_a);

	ret$me_d <- res[[1]]$me_d; 
	for (i in 2:length(res))
		ret$me_d <- c(ret$me_d, res[[i]]$me_d);

	ret$r_h2 <- res[[1]]$r_h2; 
	for (i in 2:length(res))
		ret$r_h2 <- c(ret$r_h2, res[[i]]$r_h2);
		
	return(ret);
}
