
###############################################################
# 
# Functional Genome-wide Association Studies(Dynmodel)
#
#
# Model: yi = mu + X*alpha + Z*beta + Xi*a + Zeta*d + ei
# Parameters: mu, alpha, beta, a, d, sigma
#
#  par$N: Observations: N
#  par$d1: discrete covariates: d1
#  par$d2: continuous covariates: d2
#  par$p:  SNP
#
# Y    - response, N*1
# X    - design matrix for discrete covariates, N*d1
# Z    - design matrix for continuous covariates, N*d2
# Xi   - design matrix for additive effects, N*p
# Zeta - design matrix for dominant effects, N*p
#
# References:
# 1. Jiahan Li et al. The Bayesian Lasso for Genome-wide Association 
#   Studies.
#
# History:
# 11/11/2010 Version 1.0
#
###############################################################

library(mvtnorm)


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# DYNApp.simu_test
#
# Abstract: a simpe method to call simulation
# 
# The result is saved into [simu_test_LS_$No$.rdata]
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

DYNApp.simu_test<-function(bX11Enable=FALSE)
{
	if (is.null( fGWAS_sys ) )
		fGWAS.start();

	st.par<- DYN.param();
	summary(st.par);

	st.dat<- DYN.simulate( st.par );
	summary(st.dat);
	if (bX11Enable) plot(st.dat);

	st.ret<- DYN.run( st.dat );
	summary(st.ret );
	if (bX11Enable) plot(st.ret);

	filename <- paste( "simu_test_dyn_", fGWAS_sys$seq_id(),".rdata", sep="");
	save(st.par, st.dat, st.ret,file = filename );
	
	file.report <- paste( "simu_test_dyn_", fGWAS_sys$seq_id(),".dyn.rpt.pdf", sep="");
	
	DYNApp.report(file.report, st.dat, st.ret);
	
	return( filename );
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# DYNApp.onestep
#
# Abstract: a simpe way to run a simple data set
# 
# The result is saved into "outfile.rdata"
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

DYNApp.onestep<-function(tped_file, tfam_file, sPheno_csv, file.rdata=NULL, file.report=NULL, fields=list(), bX11Enable=FALSE )
{
	if (is.null( fGWAS_sys ) )
		fGWAS.start();

	onestep.dat<- DYN.load_plink(tped_file, tfam_file, sPheno_csv, fields);
	summary(onestep.dat);
	if (bX11Enable) plot(onestep.dat);

	onestep.ret1<- DYN.run( onestep.dat );
	summary(onestep.ret1);
	if (bX11Enable) plot(onestep.ret1);

	if (is.null(file.rdata))
		file.rdata <- fGWAS.get_subfile( sPheno_csv, paste("-", fGWAS_sys$seq_id(),".dyn.rdata", sep="") );
		
	save( onestep.dat, onestep.ret1, file = file.rdata );
	
	if (is.null(file.report))
		file.report <- fGWAS.get_subfile( sPheno_csv, paste("-", fGWAS_sys$seq_id(),".dyn.rpt.pdf", sep="") );

	DYNApp.report(file.report, onestep.dat, onestep.ret1)
	
	return( file.rdata );
}

DYNApp.report<-function( pdf.file, dat, res, options=list() )
{
	Report.new( pdf.file, options );
	Report.title( "fGWAS Report(Dynamic Model)", "fGWAS Report(DYN)", "http://statgen.psu.edu" );
	Report.par( "dat.file", dat$pheno_file);
	
	Report.AddHeadline( "Data Summary", level=1 );
	frep.report_dat(dat);

	Report.AddPageBreak();
	if ( !is.null(res) )   
	{
		Report.AddHeadline( "LR2 Profile", level=1 );
		frep.report_pv_profile(dat, res) ;
	}

	Report.AddHeadline( "Significant SNPs", level=1 );
	
	if ( !is.null(res) )	  
	{
		QQ_list<-c();
		Qq_list<-c();
		qq_list<-c();
		for (i in 1:length(res$LR2.sig[,1]))
		{
			QQ_list<-c(QQ_list, sprintf("%.2f,%.2f,%.2f...", res$LR2.sig[i,14], res$LR2.sig[i,15], res$LR2.sig[i,16] ) );
			Qq_list<-c(Qq_list, sprintf("%.2f,%.2f,%.2f...", res$LR2.sig[i,18], res$LR2.sig[i,19], res$LR2.sig[i,20] ) );
			qq_list<-c(qq_list, sprintf("%.2f,%.2f,%.2f...", res$LR2.sig[i,22], res$LR2.sig[i,23], res$LR2.sig[i,24] ) );
		}

		sig.dat <- data.frame(cbind(
					name= res$snp_names[ res$LR2.sig[,1] ],
					chr = res$snp_chrs[ res$LR2.sig[,1] ], 
					pos = res$snp_pos[ res$LR2.sig[,1] ],
					LR2 = sprintf("%.2f", res$LR2.sig[,2]), 
					pv  = sprintf("%.2E", res$LR2.sig[,3]), 
					QQ  = QQ_list,
					Qq  = Qq_list,
					qq  = qq_list) );

		Report.AddTable( sig.dat, 
							 title = "Significant SNPs",
							 frame = "wang",  
							 col.names= c("Name.", "Chr.", "Pos.", "LR2", "pv",  "QQ", "Qq", "qq" ),
							 col.width= c( 0.75,    0.25,  0.75,   0.75,   0.75, 1.25, 1.25, 1.25)*254,
							 col.align= c("L",    "L",    "R",   "R",  "C",     "L",  "L",  "L"),
							 offset.x = 50, 
							 max.show = 20)

		Report.AddPageBreak();

		np.par <- list();
		for (i in 1:length(res$LR2.sig[,1]))
		{
			np.par[[i]]<- list(
						snp_name = res$snp_names[ res$LR2.sig[i,1] ],
						QQ = res$LR2.sig[ i, 14:17 ],
						Qq = res$LR2.sig[ i, 18:21 ],
						qq = res$LR2.sig[ i, 22:25 ] );
		}

		frep.report_sig_curve( np.par, length(res$LR2.sig[,1]) ) ;

	}

	Report.Output( pdf.file );
	
	return(pdf.file);
}

