
###############################################################
# 
# Functional Genome-wide Association Studies
#
#
# Model: yi = mu + X*alpha + Z*beta + Xi*a + Zeta*d + ei
# Parameters: mu, alpha, beta, a, d, sigma
#
#  par$N: Observations: N
#  par$d1: discrete covariates: d1
#  par$d2: continuous covariates: d2
#  par$p:  SNP
#
# Y    - response, N*1
# X    - design matrix for discrete covariates, N*d1
# Z    - design matrix for continuous covariates, N*d2
# Xi   - design matrix for additive effects, N*p
# Zeta - design matrix for dominant effects, N*p
#
# References:
# 1. Das K, Li J, Wang Z, Tong C, Fu G, Li Y, Xu M, Ahn K, Mauger D, Li R, Wu R.
#    A dynamic model for genome-wide association studies.
#
# History:
# 11/11/2010 Version 1.0
#
###############################################################

library(mvtnorm);

#--------------------------------------------------------------
# public: Dyn.load_plink
#
# load snp data exported by plink and CSV phenotype data for this 
# fGWAS model
# 
# input:
#  tped_file: exported by plink
#  tfam_file: exported by plink
# tpheno_file: CSV file
#
# Batch: 
#    plink --file {data_file} --recode --chr {1} --out {oufile} --transpose --noweb
#
#--------------------------------------------------------------
DYN.load_plink<-function( tped_file, tfam_file, tpheno_file, fields=list())
{
	dat <- fdat.load_plink2( tped_file, tfam_file, tpheno_file, fields);
	return( dat );
}

#--------------------------------------------------------------
# Dyn.param
#
# Create a param object for the simulation, "fGWAS.longdt.par" is returned
#--------------------------------------------------------------
DYN.param<-function()
{
	return(fdat.longdt_param());
}

#--------------------------------------------------------------
# summary(fGWAS.par.longdt)
#
# summarize a "fGWAS.par.longdt" object
#--------------------------------------------------------------
summary.fGWAS.par.longdt <- function( par )
{
	cat( fdat.summary_longdt_param(par) );
	invisible();
}

#--------------------------------------------------------------
# Dyn.simulate
#
# Simulate a data set for fGWAS, including SNP(genotype), Y(phenotype)
# covarite( X for discrete and Z for continuous )
#
# Input  
#       parameter object
# output:
#       data object
#--------------------------------------------------------------
DYN.simulate<-function( par )
{
	return( fdat.simulate_longdt(par) );
}

#--------------------------------------------------------------
# summary(fGWAS.dat.longdt)
#
# summarize the important information for the simulate data or 
# real data.
#
# input : "fGWAS.dat.longdt" object.
#--------------------------------------------------------------
summary.fGWAS.dat.longdt <- function( dat )
{
	cat( fdat.summary_dat( dat ) );
	
	plot.fGWAS.dat.longdt(dat, bOutputDoc=T);

	invisible();
}

#--------------------------------------------------------------
# private: plot(fGWAS.longdt.dat)
#
# input : "fGWAS.dat.longdt" object.
#--------------------------------------------------------------
plot.fGWAS.dat.longdt <- function( dat, bOutputDoc=FALSE )
{
	strFile <- "";
	if ( bOutputDoc )
	{
		strFile <- fGWAS.get_subfile(dat$pheno_file, paste("-", fGWAS_sys$seq_id(),".dyn.dat.pdf", sep="") );
		pdf(strFile);
	}
	
	#plot 1: tiled curves
	fplot.tiled_curves( dat$pheno_y, dat$covar_z );
	title("The curves for all individuals.");

	if ( !bOutputDoc )
		X11();

	#plot 2: overlapping curves
	fplot.overlapping_curves( dat$pheno_y, dat$covar_z );
	title("The curves for all individuals.");


	if ( bOutputDoc )
	{
		dev.off();
		return(strFile);
	}
	else
		invisible();
}

#--------------------------------------------------------------
# DYN.run
#
# 1)Perform MCMC algorithm
# 2)summarize a/d/h2
# 3)Marshal result
#--------------------------------------------------------------
DYN.run<-function( dat )
{
	genA <- dat$snps - 1 ;
	genD <- 1 - abs(genA);
	Z    <- dat$covar_z;
	Y	 <- dat$pheno_y;
	N    <- length(Y[,1]);
	P    <- length(genA[,1]);

	#inital parameter
	ymean <- mean(dat$pheno_y, na.rm=T);
	parin0 <- c(0.7, var(dat$pheno_y[,1], na.rm=T), ymean/1, ymean/10, 0.1, 0.1, ymean/2, ymean/11, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1);

	Z.min <- min( Z, na.rm=T );
	Z.max <- max( Z, na.rm=T );

	#--Create Ui matrix, conditioning on standarized age.
	all_yi <- list();
	all_zi <- list();
	for (i in 1:N)
	{
		zi.tmp <- Z[i,];
		yi.tmp <- Y[i,];

		p.missing <- which(is.na(zi.tmp));
		if ( length( p.missing )>0 )
		{
			zi.tmp <- zi.tmp[ -p.missing , drop=FALSE];
			yi.tmp <- yi.tmp[ -p.missing , drop=FALSE];
		}
		
		all_yi[[i]] <- yi.tmp ;
		all_zi[[i]] <- zi.tmp
	}

	h0 <- NA;
	try( h0<- optim( parin0, proc_hp_null, yi=all_yi, zi=all_zi, xi=dat$covar_x, method ="BFGS" ), TRUE);
	if ( any(is.na(h0)) || h0$value==0)
	{
		stop("The initial value can not fit the data set.");
	}

	n0.len <- length( parin0 );
	parin1 <- c( parin0, parin0[(n0.len-3):n0.len], parin0[(n0.len-3):n0.len] );

	LR2.res <- c();
	for(i in 1:P )
	{
		#parin1[3:22] <- parin1[3:22]*runif(12, min=0.5, max=1.5);
		parin1[1] <- 0.7;

		h1 <- NA;
		bSuccess <- TRUE;
		try( h1 <- optim( parin1, proc_hp_t0, yi=all_yi, zi=all_zi, xi=dat$covar_x, genA=genA[i,], genD=genD[i,], method = "BFGS" ), TRUE);
		if ( any(is.na(h1) ) ) 
			bSuccess <- FALSE
		else
			parin1 = h1$par;

		p.res<-c();
		if (bSuccess)
		{
			LR2 <- 2*( h0$value - h1$value );
			pv <- pchisq(LR2, df=8, lower.tail=FALSE);
			p.res <- c( LR2, pv, h1$value, h1$par );
		}
		else
			p.res <- c( NaN, NaN, rep(NaN,length(parin1)));

cat("snp", i, "=", p.res[1:10], "\n");
		
		LR2.res <- rbind(LR2.res, c(i, p.res) );
	}
	
	rownames(LR2.res)<-rownames(dat$snps);

	#-------------computing ad(significant)-------------
	sig.ord <- order( LR2.res[,2], decreasing=T);
	nTop    <- ifelse( length(sig.ord)>20, 20, length(sig.ord) );
	LR2.sig <- LR2.res[sig.ord[1:nTop],,drop=FALSE];

	#-------------marshal result--------------
	ret<-list(
		name      = "DYN",
		pheno_file= dat$pheno_file,
		snp_chrs  = dat$snps.info[,1,drop=FALSE],
		snp_pos   = dat$snps.info[,2,drop=FALSE],
		snp_names = rownames( dat$snps.info ),
		LR2.res   = LR2.res,
		LR2.sig   = LR2.sig );
	
	class(ret)<-"fGWAS.ret.dyn";
	
	return(ret);		
}

#--------------------------------------------------------------
# proc_hp_null
#
# Input  dat$
# Y    - response, N*1
# X    - design matrix for discrete covariates, N*d1
# Z    - design matrix for continuous covariates, N*d2
#
# Model: yi = mu + X*alpha + Z*beta + Xi*a + Zeta*d + ei
# Parameters: mu, alpha, beta, a, d, sigma
#
# of Observations: N
# of SNP: p
# of discrete covariates: d1
# of continuous covariates: d2
#--------------------------------------------------------------
proc_hp_null<-function( parin, yi, zi, xi )
{
	N      <- length(xi);
	rho    <- parin[1];
	sigma2 <- parin[2];
	mu_par <- parin[3:6];
	alpha_par <- parin[7:10];
	gen_par <- parin[11:14];

	if (rho>0.999 || rho<0 || sigma2<0.001)
		return(NaN);

	L.all <- 0;
	for (i in 1:N)
	{
		tp <- zi[[i]];
		m  <- length(tp);

		ui <- cbind( rep(1, length(tp)), tp, (3*tp^2-1)/2, (5*tp^3-3*tp)/2 ) ;
		all_rd <- yi[[i]] - ui%*%mu_par - ui%*%alpha_par*xi[i] - ui%*%gen_par;

		if (m>=2)
		{
			#rho.std <- rho^( tp[c(2:m)]- tp[c(1:(m-1))] );
			L <-  sum( - ( (1-rho^2) * all_rd[m]^2 + sum( (all_rd[1:(m-1)] - rho * all_rd[2:m])^2) ) );
		}
		else
			L <-  sum( - ( (1-rho^2) * all_rd[m]^2 ) );
		L  <-  L/2/sigma2/(1-rho^2)  - (m-1)*log(1-rho^2)/2 - m*log(sigma2)/2 - m/2*log(2*pi); 

		L.all <- L.all + L;
	}


	return(-L.all);
}

#--------------------------------------------------------------
# proc_hp_t0
#
# Input  dat$
# Y    - response, N*1
# X    - design matrix for discrete covariates, N*d1
# Z    - design matrix for continuous covariates, N*d2
# Xi   - design matrix for additive effects, N*p
# Zeta - design matrix for dominant effects, N*p
#
# Model: yi = mu + X*alpha + Z*beta + Xi*a + Zeta*d + ei
# Parameters: mu, alpha, beta, a, d, sigma
#
# of Observations: N
# of SNP: p
# of discrete covariates: d1
# of continuous covariates: d2
#--------------------------------------------------------------
proc_hp_t0<-function( parin, yi, zi, xi, genA, genD )
{
	N      <- length(xi);
	rho    <- parin[1];
	sigma2 <- parin[2];

	if (rho>0.999 || rho<0 || sigma2<0.001)
		return(NaN);

	mu_par <- parin[3:6];
	alpha_par <- parin[7:10];
	QQ_par <- parin[11:14];
	Qq_par <- parin[15:18];
	qq_par <- parin[19:22];

	QQ <- (genA == -1)*1;
	qq <- (genA == 1)*1;
	Qq <- (genA == 0)*1;

	L.all <- 0;
	for (i in 1:N)
	{
		tp <- zi[[i]];
		m  <- length(tp);

		ui <- cbind( rep(1, length(tp)), tp, (3*tp^2-1)/2, (5*tp^3-3*tp)/2 ) ;
		all_rd <- yi[[i]] - ui%*%mu_par - ui%*%alpha_par*xi[i] - ui%*%QQ_par*QQ[i] - ui%*%Qq_par*Qq[i] - ui%*%qq_par*qq[i];

		if (m>=2)
		{
			#rho.std <- rho^( tp[c(2:m)]- tp[c(1:(m-1))] );
			L <-  sum( - ( (1-rho^2) * all_rd[m]^2 + sum( (all_rd[1:(m-1)] - rho * all_rd[2:m])^2) ) );
		}
		else
			L <-  sum( - ( (1-rho^2) * all_rd[m]^2 ) );
		L  <-  L/2/sigma2/(1-rho^2)  - (m-1)*log(1-rho^2)/2 - m*log(sigma2)/2 - m/2*log(2*pi); 

		L.all <- L.all + L;
	}

	return(-L.all);
}

#--------------------------------------------------------------
# summary(fGWAS.ret.dyn)
#
# 1)list the significant postion
# 2)draw an overview graph for additive, dominant and heritability
#
# input : "fGWAS.ret.dyn" object.
#--------------------------------------------------------------
summary.fGWAS.ret.dyn<-function(res)
{
	stra1 <- sprintf("The significant SNPs:\n");
	stra2 <- sprintf("----------------------------------------------\n");
	stra3 <- sprintf("%5s %12s %10s %7s %7s\n", "Chr", "Name", "Position", "LR2", "p-value");
	stra4 <- c();
	for(i in 1:length(res$LR2.sig[,1]))
	{
		snpIdx <- res$LR2.sig[i,1];
		stra4 <- c( stra4, sprintf("%5.0f %12s %10.0f %7.3f %7.3f\n", 
					res$snp_chrs[snpIdx], 
					res$snp_names[snpIdx],
					res$snp_pos[snpIdx],
					res$LR2.sig[i,2], 
					res$LR2.sig[i,3])  );
	}
	stra5 <- sprintf("----------------------------------------------\n");

	strX1 <- plot.fGWAS.ret.dyn(res, bOutputDoc=TRUE);
	strX2 <- plot.fGWAS.ret.dyn2(res, bOutputDoc=TRUE);
	
	str <- paste(c(stra1,stra2,stra3,stra4,stra5,"\n", strX1, "\n", strX2, "\n"), collapse ="");
	
	cat(str);
	invisible();
}

#--------------------------------------------------------------
# plot(fGWAS.ret.dyn)
#
# draw an overview graph for additive, dominant and heritability
#
# input : "fGWAS.ret.dyn" object.
#--------------------------------------------------------------
plot.fGWAS.ret.dyn<-function(res, bOutputDoc=FALSE)
{
	strFile <- "";
	if ( bOutputDoc )
	{
		strFile <- fGWAS.get_subfile(res$pheno_file, paste("-", fGWAS_sys$seq_id(), ".dyn.man.pdf", sep=""));
		pdf(strFile);
	}
	
	pvs<- cbind( res$snp_chrs, res$LR2.res[,3] );
	rownames(pvs) <- res$snp_names;
	fplot.manhattan( pvs );

	if ( bOutputDoc )
	{
		dev.off();
		return(strFile);
	}
	else
		invisible();
}

plot.fGWAS.ret.dyn2<-function(res, bOutputDoc=FALSE)
{
	n.row <- (length(res$LR2.sig[,1])-1)%/%3 + 1;
	n.col <- 3;

	strFile <- "";
	if ( bOutputDoc )
	{
		strFile <- fGWAS.get_subfile(res$pheno_file, paste("-", fGWAS_sys$seq_id(), ".dyn.cur.pdf", sep=""));
		pdf(strFile, width=6, height=n.row*2);
	}
	
  	par(mfrow=c(n.row,n.col));

	for (i in 1:n.row )
	for (j in 1:n.col )
	{
		if ( (i-1)*3+j > length(res$LR2.sig[,1]) )
			break;

		n.par <- (i-1)*3+j;
		par(mfg=c(i, j));
		QQ.par<- res$LR2.sig[n.par ,14:17];
		Qq.par<- res$LR2.sig[n.par ,18:21];
		qq.par<- res$LR2.sig[n.par ,22:25];
		fplot.single_curve( rownames( res$LR2.sig)[n.par], QQ=QQ.par, Qq=Qq.par, qq=qq.par );
	}

	if ( bOutputDoc )
	{
		dev.off();
		return(strFile);
	}
	else
		invisible();
}

	

