###############################################################
# 
# fGWAS data unit
#
# History:
# 11/11/2010 Version 1.0
#
# Data Object:
#         fGWAS.par.longdt 
#		  fGWAS.par.nonlong 
#         fGWAS.dat.longdt 
#		  fGWAS.dat.nonlong 
# Function:
#		  fdat.longdt_param();	
#		  fdat.nonlong_param();	
#         fdat.summary_longdt_param();
#         fdat.summary_nonlong_param();
#         fdat.simulate_longdt():
#         fdat.simulate_nonlong():
#         fdat.summary_longdt_dat()
#         fdat.summary_nonlong_dat()
#         fdat.load_plink()
#         fdat.load_plink2()
#
###############################################################

library(MSBVAR);


#--------------------------------------------------------------
# fdat.longdt_param()
#
# Create a param object for the simulation, "fGWAS.par.longdt" is returned
#--------------------------------------------------------------

fdat.longdt_param<-function()
{
	par <- list();
	class(par)<- "fGWAS.par.longdt";
	
	par$model     <- "longdt";
	#par$simu_n   <- 10;
	par$simu_n    <- 600;
	# the number of total predictors
	#par$simu_p   <- 100; 
	par$simu_p    <- 5; 
	# pairwise correlation coefficient
	par$simu_rho  <- 0.4; 
	par$simu_chr  <- 1;
	
	# from Framingham data
	par$simu_mu     <- c(26.79, -6.16, 3.75, -6.39)/2; #vector, dim(t(vector))=[1,4]
	par$simu_sigma2 <- 16;
	par$simu_alpha  <- c(6, 0.29, -5.34, 6.49)/2;
	par$simu_beta   <- 0;
	
	# locations of significant SNPs, less than p
	par$simu_a_posi  <- c(1,2,3 ); 
	par$simu_a_effect<- array(0, dim=c(3,4) );
    	par$simu_a_effect[1,] <- c(2.08, 1.77, -4.11, 1.09)/2;
    	par$simu_a_effect[2,] <- c(2.34, -0.4, 1.48, -9.43)/2;
    	par$simu_a_effect[3,] <- c(2.80, -4.5, 2.00,  0.00)/2;
    	par$simu_d_posi  <- c();
	par$simu_d_posi  <- c(3,4,5);
	par$simu_d_effect<- array(0, dim=c(3,4) );
    	par$simu_d_effect[1,]<- c(2.98, -4.27, 9.64, 2.85)/2;
    	par$simu_d_effect[2,]<- c(2.09, 2.64, 3.81,  3.07)/2;
    	par$simu_d_effect[3,]<- c(2.53, -2.45, 5.42, -3.92)/2;
	
	par$simu_z <- c(30,80);
	par$simu_longdt <- c(5,12);
	
	# the number of good predictors
	pos <- c(par$simu_a_posi, par$simu_d_posi);
	par$sig_p    <- length(unique(pos)); 

	return(par);
}

#--------------------------------------------------------------
# fdat.summary_longdt_param
#
# summarize a "fGWAS.Longdt.par" object
#--------------------------------------------------------------
fdat.summary_longdt_param<- function( par )
{
	strt <- sprintf("The simulation param. for fGWAS model:\n");
	stru <- sprintf("------------------------------------\n");
	str0 <- sprintf("%15s: %s\n", 	   "Date", 		  Sys.time() );
	str1 <- sprintf("%15s: %s\n", 	   "Model",		  par$model);
	str2 <- sprintf("%15s: %-10.0f\n", "Sample size", par$simu_n );
	str3 <- sprintf("%15s: %-10.0f\n", "Chr. No",     par$chr );
	str4 <- sprintf("%15s: %-10.0f\n", "SNP count",   par$simu_p );
	str5 <- sprintf("%15s: %-10.5f\n", "rho", 		  par$simu_rho );
	str6 <- sprintf("%15s: %-10.5f\n", "sigma2", 	  par$simu_sigma2 );
	str7 <- sprintf("%15s: %-10.5f  %-10.5f  %-10.5f  %-10.5f\n", "mu",    par$simu_mu[1], par$simu_mu[2], par$simu_mu[3], par$simu_mu[4] );
	str8 <- sprintf("%15s: %-10.5f  %-10.5f  %-10.5f  %-10.5f\n", "alpha", par$simu_alpha[1], par$simu_alpha[2], par$simu_alpha[3], par$simu_alpha[4] );
	str9 <- sprintf("%15s: %d, %d, %d\n","A. Pos.",    par$simu_a_posi[1], par$simu_a_posi[2], par$simu_a_posi[3]);
	str10<- sprintf("%15s: %d, %d, %d\n","D. Pos.",   par$simu_d_posi[1], par$simu_d_posi[2], par$simu_d_posi[3]);
	str11<- sprintf("%15s: %d--%d\n","Age Range",   par$simu_z[1],    par$simu_z[2] );
	str12<- sprintf("%15s: %d--%d\n","Long Time.",  par$simu_longdt[1], par$simu_longdt[2] );
	strd <- sprintf("------------------------------------\n\n");

	str <- paste(strt, stru, str0, str1, str2, str3, str4, 
				 str5, str6, str7, str8, str9, str10, str11,
				 str12, strd, sep="" );

	return(str);
}

#--------------------------------------------------------------
# fdat.nonlong_param
#
# Create a param object for the simulation, "fGWAS.par.nonlong" is returned
#--------------------------------------------------------------
fdat.nonlong_param<-function()
{
	par <- list();
	class(par)<- "fGWAS.par.nonlong";
	
	par$model	 <- "nonlong";
	par$simu_n   <- 100;
	# the number of total predictors
	par$simu_p   <- 1000; 
	# pairwise correlation coefficient
	par$simu_rho <- 0; 
	par$simu_chr <- 1;
	
	# from Framingham data
	par$simu_mu     <- 26; 
	par$simu_sigma2 <- 3;
	par$simu_alpha  <- 0;
	par$simu_beta   <- 0;
	par$simu_Z      <- 0; 
	par$simu_X 		<- 0; 
	
	# locations of significant SNPs, less than p
	par$simu_a_posi  <- c(100,200,300,400,500,600,700,800,850,900,950 ); 
	par$simu_a_effect<- c(1.2,1.2,1.2,0.8,0.8,0.8,0.4,0.4,1.2,0.8,1.2 );
	par$simu_d_posi  <- c(50,150,250,350,450,550,650,750,850,900,950);
	par$simu_d_effect<- c(1.2,1.2,1.2,0.8,0.8,0.8,0.4,0.4,1.2,1.2,0.8);

	# the number of good predictors
	pos <- c(par$simu_a_posi, par$simu_d_posi);
	par$sig_p    <- length(unique(pos)); 

	return(par);
}


#--------------------------------------------------------------
# fdat.summary_nonlong_param
#
# summarize a "fGWAS.par.nonlong" object
#--------------------------------------------------------------
fdat.summary_nonlong_param <- function( par )
{
	strt <- sprintf("The simulation param. for fGWAS model:\n");
	stru <- sprintf("------------------------------------\n");
	str0 <- sprintf("%15s: %s\n", 	   "Date", 		  Sys.time() );
	str1 <- sprintf("%15s: %s\n", 	   "Model",		  par$model);
	str2 <- sprintf("%15s: %-10.0f\n", "Sample size", par$simu_n );
	str3 <- sprintf("%15s: %-10.0f\n", "Chr. No",     par$chr );
	str4 <- sprintf("%15s: %-10.0f\n", "SNP count",   par$simu_p );
	str5 <- sprintf("%15s: %-10.5f\n", "rho", 		  par$simu_rho );
	str6 <- sprintf("%15s: %-10.5f\n", "mu", 		  par$simu_mu );
	str7 <- sprintf("%15s: %-10.5f\n", "sigma2", 	  par$simu_sigma2 );
	str8 <- sprintf("%15s: %-10.5f\n", "alpha", 	  par$simu_alpha );
	str9 <- sprintf("%15s: %-10.5f\n", "beta", 	      par$simu_beta );
	str10 <- sprintf("%15s: %-10.5f\n", "Z", 	      par$simu_Z );
	str11 <- sprintf("%15s: %-10.5f\n","X", 	      par$simu_X );
	str12 <- sprintf("%15s: %-10.5f\n","Sig. count",  par$sig_p );
	strd <- sprintf("------------------------------------\n\n");

	str <- paste(strt, stru, str0, str1, str2, str3, str4, 
				 str5, str6, str7, str8, str9, str10, str11,
				 str12, strd, sep="" );
	cat(str);
}

#--------------------------------------------------------------
# fdat.simulate_longdt
#
# Simulate a data set for fGWAS, including SNP(genotype), Y(phenotype)
# covarite( X for discrete and Z for continuous )
#
# Input  
#       parameter object for Backcross
# output:
#       data object
#--------------------------------------------------------------
fdat.simulate_longdt<-function( par )
{
	dat<-list(
		model        = par$model,
		taskId       = fGWAS_sys$seq_id(),
		name         = "longdt",
		chr          = par$simu_chr,
		sample_times = 1,
		sample_n     = par$simu_n,
		sample_p 	 = par$simu_p,
		pheno_file   = "simu.fgwas.pheno",
		geno_file    = "simu.fgwas.geno",
		#--the follwoings are vector or matrix data.
		#--SNP raw data(include missing data and rare data)
		raw_snps     = NULL,
		#--SNP data filted by proc_rare_snp;
		snps         = NULL,
		#--individul id list
		subj_ids     = NULL,
		#--phenotype
		pheno_y      = NULL,
		#--Discrete covarite
		covar_x      = NULL,
		#--Continuous covarite
		covar_z      = NULL,
		covar_z0     = NULL);
	
	class(dat) <- "fGWAS.dat.longdt";

	set.seed( 200 );

	dat$subj_ids <- paste("sub", c(1:par$simu_n),sep="");
	#--generate snp data
	dat$raw_snps <- t( proc_simu_geno( par ) );
	dat$snps     <- dat$raw_snps;

	#-- generate SNP info
	snps.info    <- cbind(chr=1, position=c(1:par$simu_p)); 
	rownames(snps.info) <- paste("snp", c(1:par$simu_p), sep="");
	dat$snps.info<- snps.info

	#--generate phenotype data
	r.pheno      <- proc_simu_pheno.2( par, dat$raw_snps);
	dat$pheno_y  <- r.pheno[[1]];
	dat$covar_x  <- r.pheno[[2]];
	dat$covar_z  <- r.pheno[[3]];
	dat$covar_z0 <- r.pheno[[4]];
	
	st <- length( which(!is.na(dat$pheno_y)) );
   	dat$sample_times <- round(st/dat$sample_n*10)/10;

	##output to console
	cat("** Simulation is successful and data object is returned.\n");
	
	return(dat);
}

#--------------------------------------------------------------
# fdat.summary_dat(fGWAS.dat.longdt)
#
# summarize the important information for the simulate data or 
# real data.
#
# input : "fGWAS.dat.longdt" object.
#--------------------------------------------------------------
fdat.summary_dat<-function( dat )
{

	strt <- sprintf("The data set for fGWAS model:\n");
	stru <- sprintf("-----------------------------------\n");
	str0 <- sprintf("%15s: %s\n", 	 "Date", 		 Sys.time() );
	str1 <- sprintf("%15s: %s\n", 	 "Model",		 dat$model );
	str2 <- sprintf("%15s: %s\n", 	 "Pheno. file",	 dat$pheno_file );
	str3 <- sprintf("%15s: %s\n", 	 "Geno. file", 	 dat$geno_file );
	str4 <- sprintf("%15s: %-10.0f\n", "Chr. No",	 dat$chr );
	str5 <- sprintf("%15s: %-10.0f\n", "Sample size",  dat$sample_n );
	str6 <- sprintf("%15s: %-10.0f\n", "Sample times", dat$sample_times );
	str7 <- sprintf("%15s: %-10.0f\n", "SNP count",    length(dat$raw_snps[,1]) );
	str8 <- sprintf("%15s: %-10.0f\n", "Non-rare count",  length(dat$snps[,1]) );
	strd <- sprintf("------------------------------------\n\n");

	str <- paste(strt, stru, str0, str1, str2, str3, str4, 
				 str5, str6, str7, str8, strd, sep="" );
	cat(str);

	invisible();
}

#--------------------------------------------------------------
# fdat.simulate_nonlong
#
# Simulate a data set for fGWAS, including SNP(genotype), Y(phenotype)
# covarite( X for discrete and Z for continuous )
#
# Input  
#       parameter object for Backcross
# output:
#       data object
#--------------------------------------------------------------
fdat.simulate_nonlong<-function( par )
{
	dat<-list(
		model        = par$model,
		taskId       = fGWAS_sys$seq_id(),
		name         = "nonlong",
		chr          = par$simu_chr,
		sample_times = 1,
		sample_n     = par$simu_n,
		sample_p 	 = par$simu_p,
		pheno_file   = "simu.fgwas.pheno",
		geno_file    = "simu.fgwas.geno",
		#--the follwoings are vector or matrix data.
		#--SNP raw data(include missing data and rare data)
		raw_snps     = NULL,
		#--SNP data filted by proc_rare_snp;
		snps         = NULL,
		#--individul id list
		subj_ids     = NULL,
		#--phenotype
		pheno_y      = NULL,
		#--Discrete covarite
		covar_x      = NULL,
		#--Continuous covarite
		covar_z      = NULL);
	
	class(dat) <- "fGWAS.dat.nonlong";

	dat$subj_ids <- paste("sub", c(1:par$simu_n),sep="");
	#--generate snp data
	dat$raw_snps <- t( proc_simu_geno( par ) );
	dat$snps     <- dat$raw_snps;
	
	#-- generate SNP info
	snps.info    <- cbind(chr=1, position=c(1:par$simu_p)); 
	rownames(snps.info) <- paste("snp", c(1:par$simu_p), sep="");
	dat$snps.info<- snps.info

	#--generate phenotype data
	dat$pheno_y  <- proc_simu_pheno( par, dat$raw_snps);
	#--in the simulation, covarites are 0.
	dat$covar_x  <- rep(0, par$simu_n);
	dat$covar_z  <- rep(0, par$simu_n);
	
	##output to console
	cat("** Simulation is successful and data object is returned.\n");
	
	return(dat);
}

#--------------------------------------------------------------
# public: fGWAS.load_plink2
#
# load snp data exported by plink and CSV phenotype data for this 
# fGWAS model
# 
# input:
#  tped_file: exported by plink
#  tfam_file: exported by plink
# tpheno_file: CSV file
#
# Batch: 
#    plink --file {data_file} --recode --chr {1} --out {oufile} --transpose --noweb
#
#--------------------------------------------------------------
fdat.load_plink2<-function( tped_file, tfam_file, tpheno_file, fields=list())
{
	dat<-list(
		sample_times = 2,
		sample_n     = 0,
		sample_p     = 0,
		pheno_file   = tpheno_file,
		geno_file    = tped_file,
		raw_snps     = NULL,
		snps         = NULL,
		subj_ids     = NULL,
		pheno_y      = NULL,
		covar_x      = NULL,
		covar_z      = NULL,
		covar_z0     = NULL);

	class(dat)<-"fGWAS.dat.longdt";

	res.gen  <- proc_read_trans_snp(tped_file, tfam_file );
	dat$raw_snps <- res.gen$raw_snps;

	szColX="X";
	szColY="Y";
	szColZ="Z";
	if (!is.null(fields$X)) szColX <- fields$X;
	if (!is.null(fields$Y)) szColY <- fields$Y;
	if (!is.null(fields$Z)) szColZ <- fields$Z;

	res.phe  <- proc_read_pheno2(tpheno_file, szColX, szColY, szColZ, res.gen$snps)

	res2.phe <- proc_pack_pheno2( res.phe$X, res.phe$Y, res.phe$Z, res.gen$snps);

	dat$covar_z0 <- res.phe$Z;
	dat$snps.info<- res.gen$snps.info;
	dat$snps     <- res2.phe$snps;
	dat$covar_x  <- res2.phe$X;
	dat$pheno_y  <- res2.phe$Y;
	dat$covar_z  <- res2.phe$Z;
	dat$subj_ids <- rownames(res2.phe$Y);
   	dat$sample_n <- length( dat$subj_ids );
   	dat$sample_p <- length(dat$snps[,1]);

	st <- length( which(!is.na(dat$pheno_y)) );
   	dat$sample_times <- round(st/dat$sample_n*10)/10;

	gc();

  	return( dat );
}

#--------------------------------------------------------------
# public: fGWAS.load_plink
#
# load snp data exported by plink and CSV phenotype data for this 
# fGWAS model
# 
# input:
#  tped_file: exported by plink
#  tfam_file: exported by plink
# tpheno_file: CSV file
#
# Batch: 
#    plink --file {data_file} --recode --chr {1} --out {oufile} --transpose --noweb
#
#--------------------------------------------------------------
fdat.load_plink<-function( tped_file, tfam_file, tpheno_file )
{
	dat<-list(
		taskId       = fGWAS_sys$seq_id(),
		sample_times = 1,
		sample_n     = 0,
		sample_p     = 0,
		pheno_file   = tpheno_file,
		geno_file    = tped_file,
		raw_snps     = NULL,
		snps         = NULL,
		subj_ids     = NULL,
		pheno_y      = NULL,
		covar_x      = NULL,
		covar_z      = NULL);

	class(dat)<-"fGWAS.dat.nonlong";

	res <- proc_read_trans_snp(tped_file, tfam_file);
	dat$raw_snps <- res$raw_snps;
	dat$snps.info<- res$snps.info;
	dat$snps     <- res$snps;

	res <- proc_read_pheno( tpheno_file, dat$snps);
	dat$snps     <- res$snps;
	dat$pheno_y  <- c(unlist(res$phenos["Y"]));
	dat$covar_x  <- c(unlist(res$phenos["X"]));
	dat$covar_z  <- c(unlist(res$phenos["Z"]));
	dat$subj_ids <- c(unlist(res$phenos["id"]));
   	dat$sample_n <- length( dat$subj_ids );
   	dat$sample_times <- 1;
   	dat$sample_p <- length(dat$snps[,1]);
	
	gc();
	
  	return( dat );
}

#--------------------------------------------------------------
# proc_simu_geno
#
# Simulating SNPs with equal allele frequency
#--------------------------------------------------------------
proc_simu_geno<-function( par )
{
	c <- qnorm(1/4);
	
	#--simulating correlated SNPs, needs library(mvtnorm)
	sigma_x <- matrix(seq(par$simu_rho,par$simu_rho,length=par$sig_p*par$sig_p), ncol=par$sig_p)
	sigma_x <- sigma_x + diag(seq(1-par$simu_rho,1-par$simu_rho,length=par$sig_p));

	x <- rmvnorm(n=par$simu_n, mean=seq(0, 0, length=par$sig_p), sigma=sigma_x, method="chol")
	#--x has simu_n rows, and sig_p columns, check the mean and var-cov,colMeans(x),var(x)
	
	# simulating the rest of uncorrelated SNPs
	#uncorr_SNPs <- c();
	#for (i in 1:par$simu_n) 
	#	uncorr_SNPs <- rbind(uncorr_SNPs,rnorm(par$simu_p - par$sig_p, mean = 0, sd = 1))
	# bind all SNPs and convert to -1, 0, 1
	#all_SNPs <- cbind(x,uncorr_SNPs);
	
	sig_pos <- unique(c(par$simu_a_posi, par$simu_d_posi));
	all_SNPs <- c();
	for (i in 1:par$simu_p) 
	{
		fi <- which(sig_pos==i);
		if (length(fi)>0 )
			all_SNPs <- cbind(all_SNPs, x[,fi[1]])
		else
			all_SNPs <- cbind(all_SNPs, rnorm(par$simu_n, mean = 0, sd = 1))
	}	

	for (i in 1:par$simu_n) 
	{
		for (j in 1:par$simu_p) 
		{
			tmp <- all_SNPs[i,j];
			all_SNPs[i,j] <- -1*(tmp<c)+(tmp>-c);
		}
	}
	
	colnames(all_SNPs)<-paste("snp", c(1:par$simu_p),sep="");
	rownames(all_SNPs)<-paste("id", c(1:par$simu_n),sep="");

	##output to console
	cat("** Genotypical data is simulated successfully.\n");

	#--QQ:2, Qq:1, qq:0
	return (all_SNPs+1);
}

#--------------------------------------------------------------
# proc_simu_pheno
#
# Generate the phenotype data 
#--------------------------------------------------------------
proc_simu_pheno<-function( par, snp )
{
    Xi <- t(snp);
    Zeta <- 1 - abs(t(snp));
	
	# create a vector of addtive effects
    additive <- rep( 0, par$simu_p ); 
    for (i in 1:length(par$simu_a_posi) )
        additive[ par$simu_a_posi[i] ] <- par$simu_a_effect[i];

	# create a vector of dominant effects
    dominant = rep(0, par$simu_p); 
    for (i in 1:length(par$simu_d_posi) )
        dominant[ par$simu_d_posi[i] ] <- par$simu_d_effect[i];

    simu_Y <- rnorm(par$simu_n, mean=0,
    				sd=sqrt(par$simu_sigma2) ) + par$simu_X*par$simu_alpha + par$simu_Z*par$simu_beta + Xi%*%additive + Zeta%*%dominant + par$simu_mu;

	##output to console
	cat("** Phenotypical data is simulated successfully.\n");

	return(c(simu_Y));
}


#--------------------------------------------------------------
# proc_simu_pheno
#
# Generate the phenotype data 
#--------------------------------------------------------------
proc_simu_pheno.2<-function( par, snp )
{
    gen_a <- t(snp)-1;
    gen_d <- 1 - abs(gen_a);

    add_mat <- array(0, dim=c(par$simu_p, 4 ) );
    if (!is.null(par$simu_a_pos))
    for (i in 1:length(par$simu_a_pos))
    	add_mat[ par$simu_a_pos[i], ] <-par$simu_a_effect[i,];

    dom_mat <- array(0, dim=c(par$simu_p, 4 ) );
    if (!is.null(par$simu_d_pos))
    for (i in 1:length(par$simu_d_pos))
    	dom_mat[ par$simu_d_pos[i], ] <-par$simu_d_effect[i,];
    
	ind.mesu <- round(runif(par$simu_n, min=par$simu_longdt[1], max=par$simu_longdt[2]) );
	ind.Z <- array(NA, dim=c(par$simu_n, par$simu_longdt[2]) );
	for(i in 1:par$simu_n)
		ind.Z[i,1:ind.mesu[i]] <- sort( sample(par$simu_z[1]:par$simu_z[2], ind.mesu[i]) );

	ind.X <-  round(runif(par$simu_n, min=1, max=2) );
	
	ind.Y <- c();
    for(i in 1:par$simu_n)
    {
    	tmp_z <- ind.Z[i,];
    	if (length(which(is.na(tmp_z)))>0)
	        tmp_z <- tmp_z[-which(is.na(tmp_z))];

        # standarize
        tp <- -1+(2*( tmp_z - par$simu_z[1]))/( par$simu_z[2] - par$simu_z[1]);
        # create Ui matrix
        Ui <- cbind(rep(1, length(tp) ), tp, (3*tp^2-1)/2, (5*tp^3-3*tp)/2 );

        # genetic effects: mean effect plus effects of all SNPs

        gen_effect0 <- Ui %*% (par$simu_mu) + ind.X[i] * Ui %*% (par$simu_alpha);
		gen_effect <-0;
        for (j in 1:par$simu_p)
            gen_effect <-  gen_effect + gen_a[i,j]*Ui %*% (add_mat[j,]) + gen_d[i,j] * Ui %*% (dom_mat[j,]);
       
        gen_effect <- gen_effect +gen_effect0;
        # create covariance matrix
        m<-length(gen_effect);
        sigma0<-diag(m)/2;
        for (ii in 1:(m-1))
            for (jj in (ii+1):m)
                sigma0[ii,jj] <- par$simu_rho^( tmp_z[jj] - tmp_z[ii] );
        sigma <- par$simu_sigma2 * (sigma0 + t(sigma0) );

        Yi <- rmultnorm( 1, gen_effect, sigma );
        Yi <- c(Yi, rep(NA, par$simu_longdt[2] - length(Yi)) );
        ind.Y <-rbind(ind.Y, Yi);
    }
    
    ind.Z0 <- ind.Z;

	# standarize
    for(i in 1:par$simu_n)
    {
    	tmp_z <- ind.Z[i,];
        ind.Z[i,] <- -1+(2*( tmp_z - par$simu_z[1]))/( par$simu_z[2] - par$simu_z[1]);
	}

	##output to console
	cat("** Phenotypical data is simulated successfully.\n");

	return(list(ind.Y, ind.X, ind.Z, ind.Z0));
}


#--------------------------------------------------------------
# proc_rare_snp
#
# 1) remove those rare allele SNPs
# 2) simulate the missing data based on allele frequency
#
# input: snp matrix
#--------------------------------------------------------------
proc_rare_snp<-function(snps)
{
	##output to console
	set.seed(100);

	rare<-c();
	for (i in 1:length(snps[,1]) ) 
	{ 
		tmp = snps[i,]
		
		n1 = length(which(tmp==0))
		n2 = length(which(tmp==1))
		n3 = length(which(tmp==2))
		c9 = which(tmp==9)
		
		p1 = (2*n1+n2)/2/(n1+n2+n3)
		p2 = (2*n3+n2)/2/(n1+n2+n3)
		if (min(p1,p2)<0.1)
		{
			rare<-c(rare, i);
		}
		else
		{
			#simulate SNP for the missing data
			if (length(c9)>0)
			{	
				prob <- c(0, n1, n1+n2)/(n1+n2+n3); 
				for(j in c9)
				{
					missing_snp <- max( which( prob < runif(1) ) )-1;
					snps[i, j] <- missing_snp;
				}

			}
		}
	}

	if (length(rare)>0)
	{
		#how to do (snps[-rare,]) in bigMemory;
		#snps <- as.big.matrix(snps[-rare,], type="char");
		snps <- snps[-rare,];
	}

	##output to console
	cat("** Rare alleles(", length(rare), ") have been removed.\n");

	return(snps);
}

#--------------------------------------------------------------
# proc_read_trans_snp
#
# 1) load the tped and tfam file 
# 2) convert the A/C/T/G allele into 0,1,2 
#
# input: tped file/tfam file/pheno data
#--------------------------------------------------------------

proc_read_trans_snp<-function(sTped, sTfam )
{
	##output to console
	cat("** TPED file is being loaded.\n");

	tped0<-read.csv( file=sTped, sep=" ");
	snp_cnt<-length(tped0[,1]);
	snp_row<-length(tped0[1,]);
	nPerson <- (snp_row-4)/2;

	##output to console
	cat("** Genotype data transform will be performed. Individuals=", nPerson, ",SNPs=", snp_cnt, "\n");

	rawsnps <- array(0, dim=c(snp_cnt, nPerson) );

	gens<-c("C","A","T","G");
	for(i in 1:snp_cnt)
	{
		BigA<-"0";
  		for (j in 1:nPerson)
  		{
    		BigA<- tped0[i,(j*2-1)+4];
    		if (BigA!="0") break;
  		}

		if (i%%1000==1)
		{
			nSect <- 1000;
			if (i+nSect > snp_cnt)
				nSect <- snp_cnt - i + 1;
				
			tped1 <- c();
			for (j in 1:nPerson)
			{
				r = paste(tped0[i:(i+nSect-1), 4+j*2-1], tped0[i:(i+nSect-1), 4+j*2  ], sep="");
				tped1 <- cbind(tped1,r );
			}
		}

  		snpx <- array(1, nPerson );
		v0<-which(tped1[(i-1)%%1000+1,]==paste(BigA, BigA, sep=""));
		if (length(v0)>0) snpx[v0] <- 0;
		v9<-which(tped1[(i-1)%%1000+1,]=="00");
		if (length(v9)>0) snpx[v9] <- 9;
		
		gensX<- setdiff(gens, BigA);
		for (j in gensX)
		{
			v2<-which(tped1[(i-1)%%1000+1,] == paste(j,j, sep=""));
			if (length(v2)>0) snpx[v2]<-2;
		}
		
		#snps <- rbind(snps, snpx);
		rawsnps[i,] <- snpx;
		
		if (i%%1000==0)
			cat(i, memory.size(), "\n");
	}

	##output to console
	cat("** TFAM file is being loaded.\n");
	tfam0<-read.csv( file=sTfam, sep=" ", head=FALSE);
	colnames(rawsnps) <- paste(tfam0[,2], "", sep="");
	rownames(rawsnps) <- as.character( tped0[1:snp_cnt,2] );

	snps <- proc_rare_snp( rawsnps );

	snps.names <- rownames(snps);
	snps.ord <- c();
	for (i in 1:length(snps.names))
		snps.ord <- c(snps.ord, which(snps.names[i]==tped0[,2]) );

	#[,1]=chr [,2]=position
	snps.info = cbind(chr=tped0[snps.ord,1], position=tped0[snps.ord,4]); 
	rownames(snps.info) <- snps.names;

	#release memory.
	tped0<-NULL;
	gc();
	
	#0: aa 1: Aa 2:AA 9:00
	return(list(raw_snps = rawsnps , snps=snps, snps.info=snps.info ));
	
}

proc_read_pheno<-function(sPhenos, snps)
{
	##output to console
	cat("** Phenotype file is being loaded.\n");

	tpheno0<-read.csv( file=sPhenos, sep=",", head=TRUE);

	subjIds <- colnames(snps);
	tpheno<-c();
	for (i in 1:length(subjIds))
	{
		npos <- which(tpheno0[,1]==subjIds[i]);
		if (length(npos)<=0)
		{
			cat(i,"\n");
			stop("cant find genotype data for subject(",tfam0[i,2], ")\n");
		}
		tpheno<-rbind(tpheno, tpheno0[npos[1], ]);
	}

	##output to console
	cat("** Missing data in phenotype file will be removed.\n");
	ymissing <- c();

	for (i in 1:length(subjIds))
	{
		if ( tpheno[i, 2] == 0 )
			ymissing <- c(ymissing, i);
	}

	if (length(ymissing)>0)
	{
		tpheno<-tpheno[-ymissing, ];	
		snps<-snps[, -ymissing];
	}
	
	##output to console
	cat("** Phenotype and genotype data are loaded successfully.\n");

	return(list(snps=snps, phenos=tpheno));
}

proc_read_pheno2<-function(sPhenos, szColX0, szColY0, szColZ0, snps )
{
	##output to console
	cat("** Phenotype file(", sPhenos ,") is being loaded.\n");

	tpheno0<-read.csv( file=sPhenos, sep=",", head=TRUE);
	subjIds <- colnames(snps);
	tpheno<-c();
	for (i in 1:length(subjIds))
	{
		npos <- which(tpheno0[,1]==subjIds[i]);
		if (length(npos)<=0)
		{
			cat(i,"\n");
			stop("cant find genotype data for subject(",tfam0[i,2], ")\n");
		}
		tpheno<-rbind(tpheno, tpheno0[npos[1], ]);
	}

	names = colnames(tpheno0);

	pheno.X <- c();
	nColX =which(names==szColX0);
	if (length(nColX)==0)
		warning("X field is not found in the phenotype file\n")
	else
		pheno.X = tpheno[, nColX[1]];

	pheno.Y <- c();
	for(i in 1:100)
	{
		szColY = paste(szColY0, "_", i, sep="");
		nColY =which(names==szColY);
		if (length(nColY)<1)
		{
			warning(szColY,  " field is not found in the phenotype file\n")
		}
		else
		{
			cat("Y field's Name:", szColY, " index:", nColY[1], "\n");
			pheno.Y = cbind(pheno.Y, tpheno[, nColY[1]] );
		}
	}

	rownames(pheno.Y)<-subjIds;

	pheno.Z <- c();
	for(i in 1:100)
	{
		szColZ = paste(szColZ0, "_", i, sep="");
		nColZ =which(names==szColZ);
		if (length(nColZ)<1)
		{
			warning(szColZ,  " field is not found in the phenotype file\n")
		}
		else
		{
			cat("Z field's Name:", szColZ, " index:", nColZ[1], "\n");
			pheno.Z = cbind(pheno.Z, tpheno[, nColZ[1]] );
		}
	}

	rownames(pheno.Z)<-subjIds;

	if ( length(pheno.Z[1,]) != length(pheno.Y[1,]) )
		error("Z and Y dont have same length\n");

	##output to console
	cat("** Phenotype data are loaded successfully.\n");

	return(	list( X= pheno.X, Y = pheno.Y,Z=pheno.Z, id=subjIds) );
}


proc_pack_pheno2<-function( X, Y, Z, snps )
{
	N <- length(Y[,1]);
	M <- length(Y[1,]);

	y.missing <- c();
	for (i in 1:N)
	{
		z.tmp <- Z[i,];
		y.tmp <- Y[i,];

		p.missing <- union( which(y.tmp==0), which(z.tmp==0) );
		if ( length( p.missing )>0 )
		{
			y.tmp <- y.tmp[ -p.missing , drop=FALSE];
			z.tmp <- z.tmp[ -p.missing , drop=FALSE];
		}

		if ( length(p.missing) == M)
		{
			y.missing <- c( y.missing, i);
		}
		else
		{
			ord.z <- order(z.tmp);
			z.tmp <- z.tmp[ord.z];
			y.tmp <- y.tmp[ord.z];

			Z[i,] <- c(z.tmp, rep(NA, length(p.missing)) );
			Y[i,] <- c(y.tmp, rep(NA, length(p.missing)) );
		}
	}

	if (length(y.missing)>0)
	{
		Z <- Z[-y.missing,];
		Y <- Y[-y.missing,];
		X <- X[-y.missing];
		snps <- snps[ ,-y.missing ];
	}

	return(list(X=X, Y=Y, Z=Z, snps=snps));
}



#--------------------------------------------------------------
# public: fGWAS.load_snpfile2
#
# load snp data exported by simulation for this fGWAS model
# 
#--------------------------------------------------------------
fdat.load_snpfile2<-function( tsnp_file, tpheno_file, fields=list())
{
	dat<-list(
		sample_times = 2,
		sample_n     = 0,
		sample_p     = 0,
		pheno_file   = tpheno_file,
		geno_file    = tsnp_file,
		raw_snps     = NULL,
		snps         = NULL,
		subj_ids     = NULL,
		pheno_y      = NULL,
		covar_x      = NULL,
		covar_z      = NULL,
		covar_z0     = NULL);

	class(dat)<-"fGWAS.dat.longdt";
	
	tb <- read.csv(tsnp_file, header=TRUE);
	snps.names <- tb[,1];
	snps.info = cbind(chr=tb[,2], position=tb[,3]); 
	rownames(snps.info) <- snps.names;
	dat$snps.info <- snps.info;
	dim.snp       <- dim(tb);
	dat$snps      <- tb[,4:dim.snp[2]];
	dat$raw_snps  <- tb[,4:dim.snp[2]];

	tb <- read.csv(tpheno_file, header=TRUE);

	dat$covar_x  <- tb[,2];
	nZ <- (dim(tb)[2]-2)/2;

	dat$pheno_y  <- as.matrix( tb[,(nZ+3):(nZ*2+2)] );
	dat$covar_z  <- as.matrix( tb[,3:(nZ+2)] );
	dat$covar_z0 <- as.matrix( tb[,3:(nZ+2)] );
	dat$subj_ids <- c(tb[,1]);
   	dat$sample_n <- length(tb[,1]);
   	dat$sample_p <- dim(dat$snps)[1];

	st <- length( which(!is.na(dat$pheno_y)) );
   	dat$sample_times <- nZ;

	gc();

  	return( dat );
}

