###############################################################
# 
# fGWAS plot unit
#
#
# Model: yi = mu + X*alpha + Z*beta + Xi*a + Zeta*d + ei
# Parameters: mu, alpha, beta, a, d, sigma
#
# History:
# 11/11/2010 Version 1.0
#
# Function:
#		  fplot.tiled_curves();	
#		  fplot.overlapping_curves();	
#         fplot.manhattan();
#         fplot.single_adh2();
#         fplot.sig_snp_curves();
#
###############################################################


#--------------------------------------------------------------
# fplot.tiled_curves
# 
# Draw the growth curve of the individuals in some tiled plots, 
# the plot's layout can be automatically calculated.
#
# Input:
#       dat_pheno; Matrix[N, time]
#--------------------------------------------------------------
fplot.tiled_curves<-function( dat_pheno, dat_time, rows=NA, cols=NA, max_curves = NULL, selected=NULL )
{
	old.mar = par( mar = c(0,0,0,0) );
	old.mgp = par( mgp = c(0,0,0) );
	on.exit( par( old.mgp ),add = T );
	on.exit( par( old.mar ),add = T );

	max_log <- 0;
	min_log <- 0;

	py <- length ( dat_pheno[1, ] );
	sample_N <- length ( dat_pheno[,1] );

	if (is.null(colnames( dat_pheno ) ) )
		colnames(dat_pheno)<-paste( 1:py, sep="");

	if (is.null(max_curves))
		max_curves <- min(100,length(dat_pheno[,1]) );
	
	if ( is.null(selected) )
		selected <- c(1:max_curves)

	if ( is.na (rows) )
	{
		cols <- ceiling( (length(selected))^0.5 );
		rows <- ceiling( (length(selected))/cols);
	}

	minv <- min(as.matrix(dat_pheno[selected,]), na.rm=T)*0.9;
	maxv <- max(as.matrix(dat_pheno[selected,]), na.rm=T)*1.1;

	xmax <- 1;
	xmin <- -1;

	ry<- ceiling((maxv - minv)/5)
	yunit <- c(1, 2, 3, 5, 8, 10, 20, 50, 100, 200, 500, 1000, 2000);
	ry <- yunit[min(which(yunit>ry))];
	if (ry>2000) ry<-as.integer((maxv - minv)/8);


	p.width <- (xmax-xmin)*cols/9*10;
	p.height<- (maxv-minv)*rows/19*20;

	plot(c(0,0), c(0,0), type="n",xaxt="n",yaxt="n",frame.plot=FALSE, xlab="",ylab="", xlim=c(0, p.width), ylim=c(0, p.height) );
	for (i in 0:(rows-1) )
	{
		for (j in 0:(cols-1) )
		{
			sub_fig<- c( 1/cols*j, 1/cols*(j+1), 1/rows*i,  1/rows*(i+1) )*0.98+0.02;
			sub_rc <- c( p.width * sub_fig[1], p.height* sub_fig[3], 
			             p.width * sub_fig[2], p.height* sub_fig[4] );
			rect(sub_rc[1], sub_rc[2], sub_rc[3], sub_rc[4], border="black", lwd=1);
			sub_fc <- c( sub_rc[1]-(-1), sub_rc[2]-minv );

			if ( (i*cols+j+1) <= length(selected) )
			{
				idx <- i*cols+j+1;
				px<-as.numeric( dat_time[selected[idx], ] );
				py<-as.numeric( dat_pheno[selected[idx], ] );
				if (length(which(is.na(py)))>0)
				{
					px<-px[-which(is.na(py))];
					py<-py[-which(is.na(py))];
				}

				lines(px + sub_fc[1], py+sub_fc[2], lwd=1);
				
				for (k in 1:length(px))
					points(px[k]+sub_fc[1], py[k]+sub_fc[2] ,type="o", pch=19, cex=0.5)

				h<-( maxv - minv )
				if (j==0)
			  		for (k in as.integer(ceiling(minv/ry)):as.integer(floor(maxv/ry)) )
					{
						segments(-1+sub_fc[1], k*ry+sub_fc[2], -1+1/20+sub_fc[1], k*ry+sub_fc[2]);
					}

				if (i==0)
					for (k in seq(-1, 1, 0.25) )
					{
						segments( k+sub_fc[1], minv+sub_fc[2], k+sub_fc[1], minv+h/40+sub_fc[2]);
					}
			}
			else
			{
			}
			
			if (i*cols+j+1 <= sample_N)
				text( 0.7 + sub_fc[1], h*8.5/10+minv + sub_fc[2], paste(i*cols+j+1, "", sep=""), font=4);
			
			if (j==0)
			{
				for (k in as.integer(ceiling(minv/ry)):as.integer(floor(maxv/ry)) )
				{
					text(-1-0.05+sub_fc[1], k*ry+sub_fc[2], (k*ry), adj=c(1, 0.5),cex=0.5);
				}
			}

			if (i==0)
			{
				for (k in seq(-1, 0.5, 0.5) )
				{	
					text(k+sub_fc[1], minv+sub_fc[2]-h/40, k, adj=c(0.5,1),cex=0.5)
				}

				if (j==(cols-1))
					text(1+sub_fc[1], minv+sub_fc[2]-h/40, 1, adj=c(0.5,1),cex=0.5);
			}
		}
	}
	
	title(main="");
}

#--------------------------------------------------------------
# fplot.overlapping_curves
#
# Draw the growth curve of the individuals in one plots, the curves 
# will be overlapped.
#
# Input:
#       dat_pheno; Matrix[N, time]
#--------------------------------------------------------------
fplot.overlapping_curves<-function( dat_pheno, dat_time )
{
	max_log <- 0;
	min_log <- 0;

	py <- length ( dat_pheno[1, ] );

	minv <- floor( min(as.matrix( dat_pheno ), na.rm=T) )*0.9;
	maxv <- ceiling( max(as.matrix( dat_pheno ), na.rm=T) )*1.1;

	ry<- (maxv - minv)/25;
	yunit <- c(1, 2, 3, 5, 8, 10, 20, 50, 100, 200, 500, 1000, 2000);
	ry <- yunit[min(which(yunit>ry))];
	if (ry>2000) ry<-as.integer((maxv - minv)/16);
	
	old.mar <- par(mar=c(5, 4, 4, 3));
	on.exit(par(old.mar));
	
	plot(c(1:2), c(1:2), xlim=c( -1.05, 1.05 ), ylim=c(minv, maxv), 
			type="n", main="", xlab="Time", ylab="",
			xaxt="s", yaxt="s",xaxs="i", yaxs="i", bty="o");

	for (i in 1:length( dat_pheno[,1] ) )
	{
		px <- as.numeric( dat_time[i, ] );
		py <- as.numeric( dat_pheno[i, ] );
		if ( length(which(is.na(px)))>0 )
		{
			py <- py[-(which(is.na(px)))];
			px <- px[-(which(is.na(px)))];
		}

		lines(px, py, lwd=1, col="gray")
		for (k in 1:length(px))
			points(px[k], py[k], type="o", pch=19, cex=0.5,col="darkgray")
    }
}

#--------------------------------------------------------------
# fplot.manhattan
# 
# Draw 
#
# Input:rowname(pvs):snp_name
#       pvs[,1]  chromoseom no
#       pvs[,2]  p-value
#       fig.file: pdf file name
#--------------------------------------------------------------
fplot.manhattan<-function( pvs  )
{
	old.p1 <- par( xaxs="r",  yaxs="r");  # Extend axis limits by 4%
	old.p2 <- par( tck = -0.008 );
	old.p3 <- par( mgp = c(2,0.5,0) );
	old.p4 <- par( mar=c(4,4,2,2)+0.1);
	on.exit( par(old.p1),add = T )
	on.exit( par(old.p2),add = T )
	on.exit( par(old.p3),add = T )
	on.exit( par(old.p4),add = T )

	log10.pvs <- -log10( pvs[,2] );
	if ( length( which(log10.pvs>=16)) > 0 )
		log10.pvs[which(log10.pvs>=16)] <- 16;

	log10.max <- round(max(log10.pvs, na.rm=T))+1;
	if (log10.max==Inf)
		log10.max<-16;
	if (log10.max<=10)
		log10.max<-10;

	nrow <- length(log10.pvs);

	plot( 1, 1,
		  type="n", xlab="Snp", ylab=expression(-log[10](italic(p))),
		  cex.axis=0.7, xlim=c(1, nrow), ylim=c(0, log10.max ) );

	title(main="Manhattan Plot");

	cols <- c("black","red","yellow","blue","cyan","pink","green");
	points( 1:nrow, log10.pvs, pch=19, col=cols[ (pvs[,1]%%2)+1 ]  );

	p.lab <- c( -log10(0.05), seq(3, (log10.max-1), 2)  );
	abline( h= c(p.lab), col="gray", lwd=1, lty="dashed");

	text( 0, p.lab[1] + 0.1, "p=0.05", cex=0.6, adj=c(0,1) ); 
	for(i in 2:length(p.lab))
		text( 0, p.lab[i] + 0.1, paste("p=", 10^(-p.lab[i]), sep=""), cex=0.6, adj=c(0,1) ); 

	return(0);
}

#--------------------------------------------------------------
# fplot.single_adh2
# 
# Draw 
#
# Input:pdata
#
#--------------------------------------------------------------

fplot.single_adh2<-function( pdata, chr, ytitle )
{
	n.row <- length(pdata);
	ylim <- c( range(pdata)[1]*0.9, range(pdata)[2]*1.1)

	plot( c(0,0), c(0,0), type="n", xaxt="n", yaxt="s", yaxs="i", 
		  frame=T, xlab="SNP", ylab=ytitle, xlim=c(0, n.row+2), ylim=ylim );

	xlim <- c(0, length(pdata));
	nB <- length(pdata);
	snps <- c(1:nB);

    rect(snps, 0, snps+1, pdata,  col = (chr%%2+1), border =(chr%%2+1) );
}

#--------------------------------------------------------------
# fplot.single_curve
# 
# Draw 
#
# Input:pdata
#
#--------------------------------------------------------------
fplot.single_curve<-function( snp_name, QQ=NULL, Qq=NULL, qq=NULL, add=NULL, dom=NULL )
{
	old.p1 <- par( mar=c(2,2,1,1)+0.1);
	on.exit(par(old.p1),add = T);

	tp <- seq(-1, 1, 0.05);
	ui <- cbind( rep(1, length(tp)), tp, (3*tp^2-1)/2, (5*tp^3-3*tp)/2 ) ;
	
	y <- c();
	if (!is.null(QQ))  y <- cbind(y, ui%*%QQ)
	if (!is.null(Qq))  y <- cbind(y, ui%*%Qq)
	if (!is.null(qq))  y <- cbind(y, ui%*%qq)
	if (!is.null(add)) y <- cbind(y, ui%*%add)
	if (!is.null(dom)) y <- cbind(y, ui%*%dom)

	ylim.max <- max(y, na.rm=T)*1.1;
	ylim.min <- min(y, na.rm=T)*1.1;
	if (ylim.min > 0)
	   ylim.min - min(y, na.rm=T)*0.9;
	if (ylim.min>0) ylim.min = 0;

	y.num <- length(tp);

	plot( c(0,0), c(0,0), type="n", xaxt="s", yaxt="s", yaxs="i", main=snp_name, 
		  xlab="Time", ylab="Y", xlim=c(-1, 1.2), ylim=c( ylim.min, ylim.max ) );

	cur.lab <- c();
	cur.col <- c();

	if (!is.null(QQ))
	{
		y <- ui%*%QQ;
		lines(tp, y, col="red");
		cur.lab <- c( cur.lab, "QQ");
		cur.col <- c( cur.col, "red");
	}

	if (!is.null(Qq))
	{
		y <- ui%*%Qq;
		lines(tp, y, col="green");
		cur.lab <- c( cur.lab, "Qq");
		cur.col <- c( cur.col, "green");
	}

	if (!is.null(qq))
	{
		y <- ui%*%qq;
		lines(tp, y, col="blue");
		cur.lab <- c( cur.lab, "qq");
		cur.col <- c( cur.col, "blue");
	}

	if (!is.null(add))
	{
		y <- ui%*%add;
		lines(tp, y, col="orange");
		cur.lab <- c( cur.lab, "Add");
		cur.col <- c( cur.col, "orange");
	}

	if (!is.null(dom))
	{
		y <- ui%*%dom;
		lines(tp, y, col="purple");
		cur.lab <- c( cur.lab, "Dom");
		cur.col <- c( cur.col, "purple");
	}

	legend( "topright", 
				   legend = cur.lab,
	               text.width = strwidth("ABC"),
				   text.col = cur.col,
	               col = cur.col,
				   lty=1,
	               xjust = 1, 
	               yjust = 1,
				   cex=0.8)
}


fplot.blank_rect<-function( )
{
}
