
###############################################################
# 
# Functional Genome-wide Association Studies( Group Lasso)
#
#
# Model: yi = mu + X*alpha + Z*beta + Xi*a + Zeta*d + ei
# Parameters: mu, alpha, beta, a, d, sigma
#
#  par$N: Observations: N
#  par$d1: discrete covariates: d1
#  par$d2: continuous covariates: d2
#  par$p:  SNP
#
# Y    - response, N*1
# X    - design matrix for discrete covariates, N*d1
# Z    - design matrix for continuous covariates, N*d2
# Xi   - design matrix for additive effects, N*p
# Zeta - design matrix for dominant effects, N*p
#
# References:
# 1. Jiahan Li et al. The Bayesian Lasso for Genome-wide Association 
#   Studies.
#
# History:
# 11/11/2010 Version 1.0
#
###############################################################

library(mvtnorm)


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# GLSApp.simu_test
#
# Abstract: a simpe method to call simulation
# 
# The result is saved into [simu_test_LS_$No$.rdata]
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

GLSApp.simu_test<-function(bX11Enable = FALSE)
{
	if (is.null( fGWAS_sys ) )
		fGWAS.start();

	st.par<- GLS.param();
	summary(st.par);

	st.dat<- GLS.simulate( st.par );
	summary(st.dat);
	if (bX11Enable) plot(st.dat);

	st.ret<- GLS.run( st.dat );
	summary(st.ret );
	if (bX11Enable) plot(st.ret);

	st.refit<- GLS.refit( st.dat, st.ret );
	summary(st.refit );
	if (bX11Enable) plot(st.refit);

	filename <- paste( "simu_test_LS_",fGWAS_sys$seq_id(),".rdata", sep="");
	save(st.par, st.dat, st.ret, st.refit, file = filename );

	return( filename );
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# GLSApp.simu_mround
#
# Abstract: multiple simulation to get a steady result for 
#           model test and verify
# 
# The result is returned to "LS.ret" object
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

GLSApp.simu_mround<-function(nLoop, par=NULL)
{
	if (is.null(par))
		sm.par <- GLS.param();
		
	#---------multiple simulation/run------------
	rets<-list();
	for (i in 1:nLoop)
	{
		sm.dat <- GLS.simulate(sm.par);
		sm.dat$taskId<- i;
		rets[[i]] <- GLS.run(sm.dat);
		pheno_file <- sm.dat$pheno_file;
	}

	maxIter <- fGWAS.get_value("max_iter", def=2000);
	nDel <- as.integer(maxIter/10);

	fGWAS_sys$task_start( "Execute the simulation ", nLoop, " times...\n" );	

	#-------------computing a and d, h2----------
	me_as<-c();	
	me_ds<-c();	
	me_h2<-c();
	for (i in 1:nLoop)
	{
		me_a<-c();
		y<-rets[[i]]$r_a[-(1:nDel),];
		for (j in 1:length(y[1,]))
		{
			me_a<-c(me_a, median(y[,j]));
		}
		me_as<-rbind(me_as, me_a);	

		me_d<-c();
		y<-rets[[i]]$r_d[-(1:nDel),];
		for (j in 1:length(y[1,]))
		{
			me_d<-c(me_d, median(y[,j]));
		}
		me_ds<-rbind(me_ds, me_d);	

		gc();
		
		cat("Memory.size=", memory.size(), "\n");
		fGWAS_sys$task_elapsed(finished = i/nLoop, "Round:", i, "/", nLoop, ",", "$SYS_PROMPT$","\n" );
	}

	fGWAS_sys$task_stop( "The simulation test is done\n");	

	me_d <- colMeans(me_ds);
	me_a <- colMeans(me_as);

	#-------------marshel result--------------
	sm.ret <- list(model= "GLS",
 	            pheno_file = pheno_file,
	            chrs = rets[[i]]$chrs,
	            snp_names = rets[[i]]$snp_names,
	            me_a = me_a,
	            me_d = me_d);
	class(sm.ret)<-"fGWAS.gls.ret.m2";
	
	return(sm.ret);
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# GLSApp.onestep
#
# Abstract: a simpe way to run a simple data set
# 
# The result is saved into "outfile.rdata"
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

GLSApp.onestep<-function(tped_file, tfam_file, sPheno_csv, file.rdata=NULL, file.report=NULL, fields=list(), bX11Enable = FALSE)
{
	if (is.null( fGWAS_sys ) )
		fGWAS.start();

	onestep.dat<- GLS.load_plink(tped_file, tfam_file, sPheno_csv, fields);
	summary(onestep.dat);
	if (bX11Enable) plot(onestep.dat);

	onestep.ret1<- GLS.run( onestep.dat );
	summary(onestep.ret1);
	if (bX11Enable) plot(onestep.ret1);

	onestep.ret2<- GLS.refit( onestep.dat, onestep.ret1 );
	summary(onestep.ret2);
	if (bX11Enable) plot(onestep.ret2);

	if (is.null(file.rdata))
		file.rdata <- fGWAS.get_subfile( sPheno_csv, paste("-", fGWAS_sys$seq_id(),".gls.rdata", sep="") );

	save( onestep.dat, onestep.ret1, onestep.ret2, file = file.rdata );

	if (is.null(file.report))
		file.report <- fGWAS.get_subfile( sPheno_csv, paste("-", fGWAS_sys$seq_id(),".gls.rpt.pdf", sep="") );

	GLSApp.report(file.report, onestep.dat, onestep.ret1, onestep.ret2)
	
	return( file.rdata );
}

GLSApp.report<-function( pdf.file, dat, res1, res2, options=list() )
{
	Report.new( pdf.file, options );
	Report.title( "fGWAS Report(Glasso Model)", "fGWAS Report(GLS)", "http://statgen.psu.edu" );
	Report.par( "dat.file", dat$pheno_file);
	
	Report.AddHeadline( "Data Summary", level=1 );
	frep.report_dat(dat);

	Report.AddHeadline( "Effect Estimation", level=1 );
	
	if ( !is.null(res1) )   
	{
		pdata <- c();
		pdata <- cbind( pdata, res1$snp_chrs);
		pdata <- cbind( pdata, res1$snp_pos);
		pdata <- cbind( pdata, res1$mod_a);
		pdata <- cbind( pdata, res1$mod_d);
		colnames(pdata)<-c("chr", "pos", "a^2", "d^2");

		frep.report_adh2_profile( pdata, 2) ;
		Report.AddParaBreak();
	}

	Report.AddHeadline( "Significant SNPs", level=1 );
	
	if ( !is.null(res2) )	  
	{
		r <- proc_select_sigpos.2(res2);
		aList<-r$a;
		dList<-r$d;

		add_list<-c();
		dom_list<-c();
		add2_list<-c();
		dom2_list<-c();
		for (i in 1:length(aList))
		{
			k<- aList[i];
			add_sqr <- (res2$me_a[k,1]^2 + res2$me_a[k,2]^2 + res2$me_a[k,3]^2 + res2$me_a[k,4]^2);
			dom_sqr <- (res2$me_d[k,1]^2 + res2$me_d[k,2]^2 + res2$me_d[k,3]^2 + res2$me_d[k,4]^2);
			add2_list<-c(add2_list, sprintf("%.3f", add_sqr) );
			dom2_list<-c(dom2_list, sprintf("%.3f", dom_sqr) );
			add_list<-c(add_list, sprintf("%.3f,%.3f,%.3f,%.3f", res2$me_a[k,1], res2$me_a[k,2], res2$me_a[k,3], res2$me_a[k,4] ) );
			dom_list<-c(dom_list, sprintf("%.3f,%.3f,%.3f,%.3f", res2$me_d[k,1], res2$me_d[k,2], res2$me_d[k,3], res2$me_d[k,4] ) );
		}

		sig.dat <- data.frame(cbind(
					name= res2$snp_names[aList],
					chr = res2$snp_chrs[aList], 
					pos = res2$snp_pos[aList],
					add2= add2_list,
					add = add_list, 
					dom2= dom2_list,
					dom = dom_list));

		Report.AddTable( sig.dat, 
							 title = "Significant Additive SNPs",
							 frame = "wang",  
							 col.names= c("Name.", "Chr.", "Pos.", "Add.^2.", "Add.", "Dom.^2", "Dom."),
							 col.width= c( 1,      0.25,    1,      0.6,      1.9,     0.6,      1.9  )*254,
							 col.align= c("L",     "L",    "R",    "R",       "R",    "R",      "R" ),
							 offset.x = 50, 
							 max.show = 20);

		Report.AddParaBreak();

		add_list<-c();
		dom_list<-c();
		add2_list<-c();
		dom2_list<-c();
		for (i in 1:length(dList))
		{
			k<- dList[i];
			add_sqr <- (res2$me_a[k,1]^2 + res2$me_a[k,2]^2 + res2$me_a[k,3]^2 + res2$me_a[k,4]^2);
			dom_sqr <- (res2$me_d[k,1]^2 + res2$me_d[k,2]^2 + res2$me_d[k,3]^2 + res2$me_d[k,4]^2);
			add2_list<-c(add2_list, sprintf("%.3f", add_sqr) );
			dom2_list<-c(dom2_list, sprintf("%.3f", dom_sqr) );
			add_list<-c(add_list, sprintf("%.3f,%.3f,%.3f,%.3f", res2$me_a[k,1], res2$me_a[k,2], res2$me_a[k,3], res2$me_a[k,4] ) );
			dom_list<-c(dom_list, sprintf("%.3f,%.3f,%.3f,%.3f", res2$me_d[k,1], res2$me_d[k,2], res2$me_d[k,3], res2$me_d[k,4] ) );
		}

		sig.dat <- data.frame(cbind(
					name= res2$snp_names[dList],
					chr = res2$snp_chrs[dList], 
					pos = res2$snp_pos[dList],
					add2= add2_list,
					add = add_list, 
					dom2= dom2_list,
					dom = dom_list));

		Report.AddTable( sig.dat, 
							 title = "Significant Dominant SNPs",
							 frame = "wang",  
							 col.names= c("Name.", "Chr.", "Pos.", "Add.^2.", "Add.", "Dom.^2", "Dom."),
							 col.width= c( 1,      0.25,    1,     0.6,      1.9,     0.6,      1.9  )*254,
							 col.align= c("L",     "L",    "R",    "R",       "R",    "R",      "R" ),
							 offset.x = 50, 
							 max.show = 20);

		Report.AddPageBreak();
		
		np.par <- list();
		sigList<- union(aList, dList);

		for (i in 1:length(sigList))
		{
			np.par[[i]]<- list(
						snp_name = res2$snp_names[ sigList[i] ],
						add = res2$me_a[ i, 1:4 ],
						dom = res2$me_d[ i, 1:4 ] );
		}

		frep.report_sig_curve( np.par, length(sigList) ) ;

	}

	Report.Output( pdf.file );

	return(pdf.file);
}


GLSApp.onesnp2<-function(tsnp_file, sPheno_csv, file.rdata=NULL, file.report=NULL, fields=list(), bX11Enable = FALSE)
{
	if (is.null( fGWAS_sys ) )
		fGWAS.start();

	onestep.dat<- GLS.load_snpfile(tsnp_file, sPheno_csv, fields);
	summary(onestep.dat);
	if (bX11Enable) plot(onestep.dat);

	onestep.ret1<- GLS.run( onestep.dat );
	summary(onestep.ret1);
	if (bX11Enable) plot(onestep.ret1);

	onestep.ret2<- GLS.refit( onestep.dat, onestep.ret1 );
	summary(onestep.ret2);
	if (bX11Enable) plot(onestep.ret2);

	if (is.null(file.rdata))
		file.rdata <- fGWAS.get_subfile( sPheno_csv, paste("-", fGWAS_sys$seq_id(),".gls.rdata", sep="") );

	save( onestep.dat, onestep.ret1, onestep.ret2, file = file.rdata );

	if (is.null(file.report))
		file.report <- fGWAS.get_subfile( sPheno_csv, paste("-", fGWAS_sys$seq_id(),".gls.rpt.pdf", sep="") );

	GLSApp.report(file.report, onestep.dat, onestep.ret1, onestep.ret2)
	
	return( file.rdata );
}
