
###############################################################
# 
# Functional Genome-wide Association Studies(Group Lasso)
#
#
# Model: yi = mu + X*alpha + Z*beta + Xi*a + Zeta*d + ei
# Parameters: mu, alpha, beta, a, d, sigma
#
#  par$N: Observations: N
#  par$d1: discrete covariates: d1
#  par$d2: continuous covariates: d2
#  par$p:  SNP
#
# Y    - response, N*1
# X    - design matrix for discrete covariates, N*d1
# Z    - design matrix for continuous covariates, N*d2
# Xi   - design matrix for additive effects, N*p
# Zeta - design matrix for dominant effects, N*p
#
# References:
# 1. Jiahan Li et al. The Bayesian Lasso for Genome-wide Association 
#   Studies.
#
# History:
# 11/11/2010 Version 1.0
#
###############################################################

library(MSBVAR);

new_solve<-function(A)
{
	return( chol2inv(chol(A)) )
	#return( solve(A) )
}

#--------------------------------------------------------------
# public: GLS.load_plink
#
# load snp data exported by plink and CSV phenotype data for this 
# fGWAS model
# 
# input:
#  tped_file: exported by plink
#  tfam_file: exported by plink
#  tpheno_file: CSV file
#
# Batch: 
#    plink --file {data_file} --recode --chr {1} --out {oufile} --transpose --noweb
#
#--------------------------------------------------------------
GLS.load_plink<-function( tped_file, tfam_file, tpheno_file, fields=list())
{
	dat <- fdat.load_plink2( tped_file, tfam_file, tpheno_file, fields);
	return( dat );
}

#--------------------------------------------------------------
# public: GLS.load_plink
#
# load snp data exported by plink and CSV phenotype data for this 
# fGWAS model
# 
# input:
#  tped_file: exported by plink
#  tfam_file: exported by plink
#  tpheno_file: CSV file
#
# Batch: 
#    plink --file {data_file} --recode --chr {1} --out {oufile} --transpose --noweb
#
#--------------------------------------------------------------
GLS.load_snpfile<-function( tsnp_file, tpheno_file, fields=list())
{
	dat <- fdat.load_snpfile2( tsnp_file, tpheno_file, fields);
	return( dat );
}

#--------------------------------------------------------------
# GLS.param
#
# Create a param object for the simulation, "fGWAS.par.longdt" is returned
#--------------------------------------------------------------
GLS.param<-function()
{
	return(fdat.longdt_param());
}

#--------------------------------------------------------------
# summary(fGWAS.longdt.par)
#
# summarize a "fGWAS.longdt.par" object
#--------------------------------------------------------------
summary.fGWAS.par.longdt <- function( par )
{
	cat( fdat.summary_longdt_param(par) );
	invisible();
}

#--------------------------------------------------------------
# GLS.simulate
#
# Simulate a data set for fGWAS, including SNP(genotype), Y(phenotype)
# covarite( X for discrete and Z for continuous )
#
# Input  
#       parameter object
# output:
#       data object
#--------------------------------------------------------------
GLS.simulate<-function( par )
{
	return( fdat.simulate_longdt(par) );
}

#--------------------------------------------------------------
# summary(fGWAS.dat.longdt)
#
# summarize the important information for the simulate data or 
# real data.
#
# input : "fGWAS.dat.longdt" object.
#--------------------------------------------------------------
summary.fGWAS.dat.longdt <- function( dat )
{
	cat( fdat.summary_dat( dat ) );

	plot.fGWAS.dat.longdt( dat, bOutputDoc=T );
	invisible();
}

#--------------------------------------------------------------
# private: plot(fGWAS.longdt.dat)
#
# input : "fGWAS.dat.longdt" object.
#--------------------------------------------------------------
plot.fGWAS.dat.longdt <- function( dat, bOutputDoc=FALSE )
{
	strFile <- "";
	if ( bOutputDoc )
	{
		strFile <- fGWAS.get_subfile(dat$pheno_file, paste("-", fGWAS_sys$seq_id(),".gls.dat.pdf", sep=""));
		pdf(strFile);
	}
	
	#plot 1: tiled curves
	fplot.tiled_curves( dat$pheno_y, dat$covar_z );
	title("The curves for all individuals.");

	if ( !bOutputDoc )
		X11();

	#plot 2: overlapping curves
	fplot.overlapping_curves( dat$pheno_y, dat$covar_z );
	title("The curves for all individuals.");

	if ( bOutputDoc )
	{
		dev.off();
		return(strFile);
	}
	else
		invisible();
}

#--------------------------------------------------------------
# GLS.run
#
# 1)Perform MCMC algorithm
# 2)summarize a/d/h2
# 3)Marshal result
#--------------------------------------------------------------
GLS.run<-function( dat, refit=FALSE )
{
	seed.n <- Sys.getpid() + as.numeric(format(Sys.time(), "%H%M%S"))
	seed.n <- 100;
	cat("!!!Set.seed=", seed.n, "\n");
	set.seed( seed.n );

	#-------------MCMC algorithm-------------
	#Data Feeding and Cleaning, load univariate response 
	r_a       <- c();
	r_d       <- c();
	r_mu      <- c();
	r_alpha   <- c();
	r_sigma2  <- c();

	p <- length(dat$snps[,1]);
	res <- c();
	i.step<-10000;

	for(i in 1:ceiling(p/i.step ) )
	{
		if (i*i.step<=p)
		{
			Xi.sect   <- dat$snps[((i-1)*i.step+1):(i*i.step),] - 1 ;
			Zeta.sect <- 1-abs(Xi.sect);
		}
		else
		{
			Xi.sect   <- dat$snps[((i-1)*i.step+1):p,] - 1 ;
			Zeta.sect <- 1-abs(Xi.sect);
		}
		
		res.sect <- proc_mcmc.2(dat$pheno_y, dat$covar_x, dat$covar_z, dat$covar_z0,t(Xi.sect), t(Zeta.sect), taskId=dat$taskId, sectId=i, refit=refit );
		
		r_a       <- cbind(r_a,     res.sect$r_a );		#dim(r_a)=[4*R, p]
		r_d       <- cbind(r_d,     res.sect$r_d );		#dim(r_d)=[4*R, p]
		r_alpha   <- cbind(r_alpha, res.sect$r_alpha );
		r_mu      <- cbind(r_mu,    res.sect$r_mu );
		r_sigma2  <- cbind(r_sigma2,res.sect$r_sigma2 );
		
		gc();
	}

	#-------------computing a(additive)-------------
	nDel <- as.integer(fGWAS.get_value("max_iter", def=2000)/10);
	y <- r_a[-(1:(nDel*4)),];
	sum<-array(0, dim=c(4, length(y[1,])));
	for (i in  1:(length(y[,1])/4) )
	{
		tmp = y[(i*4-3):(i*4),];
		sum <- sum + tmp;
	}
	mean_a = sum/(length(y[,1])/4);

	mod_a <- c();
	for(i in 1:length(mean_a[1,]))
	   mod_a <-c (mod_a, mean_a[1,i]^2 + mean_a[2,i]^2 + mean_a[3,i]^2 + mean_a[4,i]^2 );

	#-------------computing d(dominant)-------------
	y <- r_d[-(1:(nDel*4)),];
	sum<-array(0, dim=c(4, length(y[1,])));
	for (i in  1:(length(y[,1])/4) )
	{
		tmp = y[(i*4-3):(i*4),];
		sum <- sum + tmp;
	}
	mean_d <- sum/(length(y[,1])/4);

	mod_d <- c();
	for(i in 1:length(mean_d[1,]))
	   mod_d <-c (mod_d, mean_d[1,i]^2 + mean_d[2,i]^2 + mean_d[3,i]^2 + mean_d[4,i]^2 );

	#-------------marshal result--------------
	ret<-list(
		model     = "GLS",
		name      = "gls.vs",
		taskId    = fGWAS_sys$seq_id(),
		pheno_file= dat$pheno_file,
		snp_chrs  = dat$snps.info[,1,drop=FALSE],
		snp_pos   = dat$snps.info[,2,drop=FALSE],
		snp_names = rownames(dat$snps),
		me_a      = mean_a,
		me_d      = mean_d,
		mod_a     = mod_a,
		mod_d     = mod_d );
	
	class(ret)<-"fGWAS.ret.gls";
	
	return(ret);		
}

#--------------------------------------------------------------
# proc_mcmc.2
#
# Input 
# Y    - response, N*1
# X    - design matrix for discrete covariates, N*d1
# Z    - design matrix for continuous covariates, N*d2
# Xi   - design matrix for additive effects, N*p
# Zeta - design matrix for dominant effects, N*p
#
# Model: yi = mu + X*alpha + Z*beta + Xi*a + Zeta*d + ei
# Parameters: mu, alpha, beta, a, d, sigma
#
# of Observations: N
# of SNP: p
# of discrete covariates: d1
# of continuous covariates: d2
#--------------------------------------------------------------
proc_mcmc.2<-function( Y, X, Z, Z0, gen_a, gen_d, taskId=-1, sectId=-1, refit=FALSE  )
{
#refit <- TRUE;
#gen_a <- gen_a[,1:10];
#gen_d <- gen_d[,1:10];

	maxIter <- as.integer( fGWAS.get_value("max_iter", def=2000) );
	mcmc_hint <- as.integer( fGWAS.get_value("mcmc_hint", def=100) );
	rho_tuning <- 0.05;
	cat("!!!max_iter=", maxIter, "rho_tuning=", rho_tuning, "\n");

	#--check dimensions
	N <- length(Y[,1]);
	P <- length(gen_a[1,]);
	d1<- 1;
	d2<- 1;
	sum_Ji <- length(which(Y!=0)|which(!is.na(Y)));

	#--Hyper-parameters for alpha, beta, a, d, and lambda2
	sigmaMu    <- diag(rep(1, 4), nrow=4)
	sigmaAlpha <- diag(rep(1, 4), nrow=4);
	#sigmaBeta  <- diag(rep(1, d2), nrow=d2);
	gammaTau   <- 1;
	deltaTau   <- 1;
	gammaTau_st <- 1;
	deltaTau_st <- 1;
	
	#--Initializing all parameters: mu, alpha, beta, sigma2, a, d, ...
	sigma2 <- 8;
	alpha <- array(0, dim=c(d1,4));
	beta  <- rep(0, d2);
	#--additive effect and its hyperparameters
	a     <- array(0, dim=c(P, 4)); 
	tau2  <- rep( 1, P);
	lambda2 <- 1;
	#--dominant effect and its hyperparameters
	d     <- array(0, dim=c(P, 4)); 
	tau_st2 <- rep(1, P);
	lambda_st2 <- 1;
	rho  <- 0.4;

	
	r_sigma2  <- c();
	r_mu      <- c();
	r_alpha   <- c();
	r_a       <- c(); 
	r_tau2    <- c();
	r_lambda2 <- c();
	r_d       <- c(); 
	r_tau_st2 <- c();
	r_lambda_st2 <- c();


	## output to console
	cat("** MCMC sampling procedure will start", "Section:", sectId, ".\n");
	
	Z.min <- min( Z, na.rm=T );
	Z.max <- max( Z, na.rm=T );

	#--Create Ui matrix, conditioning on standarized age.
	all_ui <- list();
	all_yi <- list();
	all_rd <- list();
	for (i in 1:N)
	{
		zi.tmp <- Z[i,];
		yi.tmp <- Y[i,];

		p.missing <- which(is.na(zi.tmp));
		if ( length( p.missing )>0 )
		{
			zi.tmp <- zi.tmp[ -p.missing , drop=FALSE];
			yi.tmp <- yi.tmp[ -p.missing , drop=FALSE];
		}
		
		if (Z.max > 1 || Z.min < -1)
			tp <- -1 + 2*(zi.tmp-Z.min)/(Z.max-Z.min)
		else
			tp <- zi.tmp;

		all_ui[[i]] <- cbind( rep(1, length(tp)), tp, (3*tp^2-1)/2, (5*tp^3-3*tp)/2 ) ;
		all_yi[[i]] <- yi.tmp ;
		all_rd[[i]] <- yi.tmp ;
	}

	fGWAS_sys$task_start( 1, maxIter );

	#--begin Gibbs sampling
	for (round in 1:maxIter)
	{	
		if ((round%%mcmc_hint==0) || (round<10 ) )
		{
			msg <- paste("Section:", sectId, " Task.id:", taskId, " Round:", round, " Estimation: $SYS_PROMPT$", sep="");
			fGWAS_sys$task_elapsed( round, msg=msg);
		}

		#-- Calculating correlation matrix
	    	# new proposal of rho
    		tmp5 <- runif( 1, min=max(-1,rho-rho_tuning),max = min(1,rho+rho_tuning) );
	
		all_corMat<- list();
	    	all_corMat_MH <- list();
	    	for (i in 1:N)
		{
			#-- create covariace matrix, conditioning on rho, sigma2
			m <- length( which( !is.na(Z[i,]) ) );
			if (m==0)
			{
				cat("no phenotype data in subject(", i, ")\n");
				browser();
			}

			sigma0  <- diag(m)/2;
			sigma00 <- diag(m)/2;
			if (m>1)
			for (ii in 1:(m-1) )
			{
				for (jj in (ii+1):m)
				{	
					sigma0 [ii,jj] <- rho^(  Z0[i,jj] - Z0[i,ii] );
					sigma00[ii,jj] <- tmp5^( Z0[i,jj] - Z0[i,ii] );
				}
			}

			# correlation
			all_corMat[[i]] <- ( sigma0 + t(sigma0) ); 
			# correlation based on proposed rho
			all_corMat_MH[[i]] <- ( sigma00 + t(sigma00) ); 
	    	}

		#--- Updating mu
		tmp2 <- 0;
		tmp3 <- 0;
		for (i in 1:N)
		{
			# calculate mean and variance
			ui <- all_ui[[i]];
			sigma <- sigma2 * all_corMat[[i]];
			tmp  <- new_solve(sigma) %*% ui;
			tmp3 <- tmp3 + ( all_rd[[i]] - X[i]*alpha[1,]%*%t(ui) ) %*% tmp;
			tmp2 <- tmp2 + t(ui) %*% tmp;
		}

		
		Mu_Var_j <- new_solve( new_solve(sigmaMu) + tmp2);
		Mu_Mu_j  <- Mu_Var_j %*% t(tmp3);
		mu <- rmultnorm( 1, Mu_Mu_j,Mu_Var_j);

		#--- Updating beta 
		#%     betaVar = 1/(Z'*Z/sigma2+1/sigmaBeta);
		#%     betaMu = betaVar*(Z'*(Y-X*alpha-Xi*a'-Zeta*d')/sigma2);
		#%     beta = normrnd(betaMu,sqrt(betaVar));

		#-- Calculating effect other than genetic effects: yi-\mu_i after updating a, d 
		
		for (i in 1:N)   
		{
			mean_effect <- 0;
			for (jj in 1:P)
			{
				mean_effect <- mean_effect + gen_a[i,jj]*all_ui[[i]]%*%(a[jj,]) + gen_d[i,jj] * all_ui[[i]] %*% (d[jj,]);
			}
			
			all_rd[[i]] <- all_yi[[i]] - t(mean_effect);
		}
		
		# record the "a" used to construct all_rd{i}
		a_old <- a; 
		# record the "d" used to construct all_rd{i}
		d_old <- d; 

		#-- Updating additive effects: a, for each SNP
		for (j in 1:P)
		{
			tmp2 <- 0;
			tmp3 <- 0;
			for (i in 1:N)
			{
				ui <- all_ui[[i]];
				sigma <- sigma2 * all_corMat[[i]];

				# calculate mean and variance
				tmp  <- (new_solve(sigma) * gen_a[i,j]) %*% ui;
				tmp3 <- tmp3 + (all_rd[[i]] - mu%*%t(ui) - X[i]*alpha[1,]%*%t(ui) + gen_a[i,j]*a[j,]%*%t(ui))%*%tmp;
				tmp2 <- tmp2 + gen_a[i,j]*t(ui)%*%tmp;
			}
			if (refit)
				aVar_j<- new_solve( tmp2 )
			else
				aVar_j<- new_solve( 1/sigma2 * diag(rep(1/tau2[j],4), nrow=4) + tmp2 );

			aMu_j <- aVar_j%*%t(tmp3);
			a[j,] <- rmultnorm( 1, aMu_j, aVar_j );

			# Update all_resid{i} for the newly updated genetic effect
			for (i in 1:N)  
				all_rd[[i]] <- all_rd[[i]] + t( gen_a[i,j]*all_ui[[i]]%*%(a_old[j,]) - gen_a[i,j]*all_ui[[i]]%*%(a[j,]) );
		}

		#-- Updating tau2 underlying a
		if (!refit)
		{
			for (j in 1:P)
			{
				InvTau2_1 <- sqrt( 4 * lambda2 * sigma2/(t(a[j,])%*%(a[j,]) ) );
				tau2[j]   <- 1/func_invGau( InvTau2_1, 4*lambda2 );
			}

			#-- Updating lambda2 underlying tau2
			lambda2 <- rgamma( 1, (4*P+P)/2+gammaTau, scale=2*sum(tau2)+deltaTau );
		}
        	else
            		lambda2 <-0;

		#-- Updating dominant effects: d, for each SNP
		for(j in 1:P)
		{
			tmp2 <- 0;
			tmp3 <- 0;
			for(i in 1:N)
			{
				ui <- all_ui[[i]];
				sigma <- sigma2 * all_corMat[[i]];

				# calculate mean and variance
				tmp  <- (new_solve(sigma) * gen_d[i,j]) %*% ui;
				tmp3 <- tmp3 + ( all_rd[[i]] - mu%*%t(ui) - X[i]*alpha[1,]%*%t(ui) + gen_d[i,j]*d[j,]%*%t(ui))%*%tmp;
				tmp2 <- tmp2 + gen_d[i,j]*t(ui)%*%tmp;
			}

			if (refit)
				dVar_j <- new_solve( tmp2)
			else
				dVar_j <- new_solve( 1/sigma2*diag(rep(1/tau_st2[j], 4), nrow=4) + tmp2);
		
			dMu_j  <- dVar_j%*%t(tmp3);
			d[j,]  <- rmultnorm( 1, dMu_j, dVar_j);

			# Update all_resid{i} for the newly updated genetic effect
			for (i in 1:N)
				all_rd[[i]] <- all_rd[[i]] + t(gen_d[i,j]*all_ui[[i]]%*%(d_old[j,]) - gen_d[i,j]*all_ui[[i]]%*%(d[j,]) );
		}

		if (!refit)
		{
			#-- Updating tau_st2 underlying d
			for (j in 1:P)
			{
				InvTau2_1  <- sqrt( 4 * lambda_st2 * sigma2/(t(d[j,])%*%(d[j,])));
				tau_st2[j] <- 1/func_invGau(InvTau2_1, 4*lambda_st2);
			}


			#-- Updating lambda_st2 underlying tau_star2
			lambda_st2 <- rgamma( 1, (4*P+P)/2+gammaTau_st, scale=2*sum(tau_st2)+deltaTau_st );
		}
        	else
            		lambda_st2 <-0;

		#-- Updating single covariates: alpha
		tmp2 <- 0;
		tmp3 <- 0;
		for (i in 1:N)
		{
			# calculate mean and variance
			ui   <- all_ui[[i]];
			sigma<- sigma2 * all_corMat[[i]];
			tmp  <- (new_solve(sigma) * X[i]) %*% ui;
			tmp3 <- tmp3 + ( all_rd[[i]] - mu%*%t(ui) )%*%tmp;
			tmp2 <- tmp2 + X[i]*t(ui)%*%tmp;
		}

		
		alpha_Var_j<- new_solve( new_solve ( sigmaAlpha ) + tmp2 );
		alpha_Mu_j <- alpha_Var_j %*% t(tmp3);
		alpha[1,]  <- rmultnorm( 1, alpha_Mu_j, alpha_Var_j );

		
		#-- Updating residual variance: sigma2 
		sigma2_scale <- 0;
		for(i in 1:N)
		{
			# calculate scale parameter
            		tmp_sig <- all_rd[[i]] - mu%*%t( all_ui[[i]] ) - X[i]*alpha[1,]%*%t(all_ui[[i]]) 
			sigma2_scale <- sigma2_scale + (tmp_sig)%*%new_solve( all_corMat[[i]])%*%t(tmp_sig );
		}
		sigma2_scale <- sigma2_scale/sum_Ji;
		sigma2 <- 1/rchisq(1, sum_Ji)*sum_Ji*sigma2_scale;
		sigma2 <- sigma2[1,1];


		#-- Updating rho, proposal distribution Q
		Qnew <- dunif( tmp5, max(-1,rho-rho_tuning), min(1,rho+rho_tuning) , FALSE ); 
		Qold <- dunif( rho, max(-1,tmp5-rho_tuning), min(1,tmp5+rho_tuning), FALSE );
		#if (is.na(1/Qnew))
		#	Qnew <- dunif( tmp5, max(-1,rho-rho_tuning), min(1,rho+rho_tuning) ); 
		
		# target density with new sample
		det_diff <- 1;
		exp_diff <- 0;
		for (i in 1:N)
		{
			sigma_old <- sigma2*all_corMat[[i]];
			sigma_new <- sigma2*all_corMat_MH[[i]];
			ui <- all_ui[[i]];
			# calculate scale parameter  
			allRd_i <- all_rd[[i]] - mu%*%t(ui) - X[i]*alpha[1,]%*%t(ui);
			exp_diff   <- exp_diff + allRd_i %*% new_solve(sigma_new) %*% t(allRd_i) - allRd_i %*% new_solve(sigma_old) %*% t(allRd_i);
			det_diff   <- det_diff * sqrt(det(sigma_old)/det(sigma_new));  
		}

		Pnew_Pold <- det_diff*exp(-exp_diff/2);
		accpp <-min(1,Pnew_Pold*Qold/Qnew);
		tmp6 <- runif(1);
		rho  <- tmp5*(tmp6<accpp)+rho*(tmp6>=accpp);

		#-- save results of current round
		r_sigma2  <- rbind( r_sigma2, sigma2); 	#dim(r_sigma2)=1 
		r_alpha   <- rbind( r_alpha, alpha);   	#dim(alpha)=[1,4]
		r_mu      <- rbind( r_mu, mu);	   		#dim(beta)=1
		r_a       <- rbind( r_a, t(a) );   		#dim(a)=[p,4], so dim(r_a) = [4*R, p]
		r_d       <- rbind( r_d, t(d) );   		#dim(d)=[p,4]

		cat("round[", round, "] sigma2=", sigma2, " rho=", rho, " tmp5=", tmp5, " lambda2=", lambda2, " lambda_st2=", lambda_st2, " mu=", mu,"\n");
	}

	ret<-list(
		r_alpha   = r_alpha,
		r_mu      = r_mu,
		r_a       = r_a,
		r_d       = r_d,
		r_sigma2  = r_sigma2 );
	
	##output to console
	fGWAS_sys$task_stop("** MCMC sampling procedure is done successfully.");

	return( ret )
}

func_invGau<-function(theta,chi)
{
	# squared normal, i.e., chi-square with df=1
	chisq1<- rnorm(1, 0, 1)^2; 
	y1    <- theta + 0.5*theta/chi * ( theta*chisq1 - sqrt(4*theta*chi*chisq1 + theta^2*chisq1^2) );
	y2    <- theta^2/y1;
	out_1 <- runif(1) < theta/(theta+y1);
	value <- out_1*y1+(1-out_1)*y2;
	
	return(value);
}

#--------------------------------------------------------------
# GLS.refit
#
#
# input: fGWAS.dat.longdt and fGWAS.ret.gls
#--------------------------------------------------------------

GLS.refit<-function(org_dat, org_res)
{
	##output to console
	cat("** Significant positions is being selected for Beyansian Lasso Model again\n");
	r <- proc_select_sigpos.2(org_res);
	rList <- sort(unique(c(r$a, r$d)));
	
	dat<-list(
		model        = "GLS",
		sample_times = 1,
		sample_n     = 0,
		sample_p     = 0,
		pheno_file   = org_dat$pheno_file,
		geno_file    = org_dat$geno_file,
		raw_snps     = NULL,
		snps         = NULL,
		subj_ids     = org_dat$subj_ids,
		pheno_y      = org_dat$pheno_y,
		covar_x      = org_dat$covar_x,
		covar_z      = org_dat$covar_z,
		covar_z0     = org_dat$covar_z0);
	
	dat$snps.info<- org_dat$snps.info[rList,,drop=FALSE];
	dat$snps     <- org_dat$snps[rList,,drop=FALSE];
	dat$sample_n <- length(dat$subj_ids);
	dat$sample_p <- length(dat$snps[,]);
	class(dat) <- "fGWAS.dat.longdt";

	ret <- GLS.run(dat, refit=TRUE);

	class(ret) <- "fGWAS.ret.gls";
	ret$name   <- "gls.refit"

	cat("** Refit procedure is done successfully\n");
	
	return(ret);		
}

#--------------------------------------------------------------
# summary(fGWAS.ret.gls)
#
# 1)list the significant postion
# 2)draw an overview graph for additive, dominant and heritability
#
# input : "fGWAS.ret.gls" object.
#--------------------------------------------------------------
summary.fGWAS.ret.gls<-function( res )
{
	r<-proc_select_sigpos.2(res);
	aList <- r$a;
	dList <- r$d;
	
	stra1 <- sprintf("The additive significant SNPs:\n");
	stra2 <- sprintf("----------------------------------------------\n");
	stra3 <- sprintf("%5s %12s %10s %7s %7s\n", "Chr", "Name", "Position", "Additive(Mod)", "Dominant(Mod)");
	stra4 <- c();
	for(i in 1:length(aList))
	{
		stra4 <- c(stra4, sprintf("%5.0f %12s %10.0f %7.3f %7.3f\n", 
					res$snp_chrs[aList[i]], 
					res$snp_names[aList[i]], 
					res$snp_pos[aList[i]], 
					res$mod_a[aList[i]], 
					res$mod_d[aList[i]]) );
	}
	stra5 <- sprintf("----------------------------------------------\n");


	strd1 <- sprintf("The dominant significant SNPs:\n");
	strd2 <- sprintf("----------------------------------------------\n");
	strd3 <- sprintf("%5s %12s %10s %7s %7s\n", "Chr", "Name", "Position", "Dominant(Mod)", "Additive(Mod)");
	strd4 <- c();
	for(i in 1:length(dList))
	{
		strd4 <- c(strd4, sprintf("%5.0f %12s %10.0f %7.3f %7.3f\n", 
					res$snp_chrs[dList[i]], 
					res$snp_names[dList[i]], 
					res$snp_pos[dList[i]], 
					res$mod_d[dList[i]], 
					res$mod_a[dList[i]] ) );
	}
	strd5 <- sprintf("----------------------------------------------\n");
	
	strX <- plot.fGWAS.ret.gls(res, bOutputDoc=TRUE);
	
	str <- paste(c(stra1,stra2,stra3,stra4,stra5,"\n", 
		 		 strd1,strd2,strd3,strd4,strd5,"\n", strX), collapse ="");
	
	cat(str);
}

proc_select_sigpos.2<-function(res)
{
	absRatio <- as.numeric( fGWAS.get_value("AbsRatio", 0.1) );

	#select additive SNPs
	aList <- which( res$mod_a > absRatio );
	if (length(aList)>10 || length(aList)==0 )	
	{
		y <- order(res$mod_a, decreasing=TRUE);

		if ( length(aList)>10) 
			n.Top <- 10
		else
			n.Top<-length(res$mod_a);

		aList <- y[1:n.Top];
	}

	#select dominant SNPs
	dList <- which( res$mod_d > absRatio );
	if (length(dList)>20 || length(dList)==0 )	
	{
		y <- order(res$mod_d, decreasing=TRUE);

		if ( length(dList)>10) 
			n.Top <- 10
		else
			n.Top<-length(res$mod_d);

		dList <- y[1:n.Top];
	}
	
	return (list(a=aList, d=dList));
}

#--------------------------------------------------------------
# plot(fGWAS.ret.gls)
#
# draw an overview graph for additive, dominant and heritability
#
# input : "fGWAS.ret.gls" object.
#--------------------------------------------------------------

plot.fGWAS.ret.gls<-function(res, bOutputDoc=FALSE)
{
	strFile = "";
	if ( bOutputDoc )
	{
		strFile <- fGWAS.get_subfile( res$pheno_file, paste("-", fGWAS_sys$seq_id(),".", res$name,".pdf", sep=""));
		pdf( strFile );
	}
	
	if ( res$name== "gls.vs" )
	{
		par(mar=c(4.5, 4, 0.5, 2) + 0.1);
		plot.new();
		par(mfrow=c(2,1));

		par(mfg=c(1, 1));
		fplot.single_adh2( res$mod_a, res$snp_chrs, "Estimated additive effect" );
		par(mfg=c(2, 1));
		fplot.single_adh2( res$mod_d, res$snp_chrs, "Estimated dominant effect" );
	}
	else
	{
		r<-proc_select_sigpos.2(res);
		sig.list <- union(r$a, r$d);

		n.row <- (length(sig.list)-1)%/%3 + 1;
		n.col <- 3;
		par(mfrow=c(n.row,n.col));

		for (i in 1:n.row )
		for (j in 1:n.col )
		{
			if ( i*3+j > length(res$LR2.sig[,1]) )
				break;

			n.par <- i*3+j;
			par(mfg=c(i, j));
			add.par<- res$me_a[ sig.list[i] ];
			dom.par<- res$me_d[ sig.list[i] ];
			snp.name<- res$snp_names[ sig.list[i] ];
			fplot.single_curve( snp.name, add=add.par, dom=dom.par );
		}
	}

	if (bOutputDoc )
	{
		dev.off();
		str <- paste("*1:The figure 1 for all SNPs is saved to ", strFile, ".\n", sep="");
		return (str);
	}
}




